unit stats;

interface

uses
	crt,
	putils, slenter, strings, strznnp, data, defs, dtime,
	filedef, general;

procedure StatsBBS;
procedure StatsNode;
procedure StatsGroup;


implementation

var
	statfile 		: file;
	mainstats 		: _mainstats;

	fptr    : text;
	date	: datetype;
	d		: time_t;
	ns,
	total	: _nodestats;
	sz		: sysznnp;
	found	: boolean;


function OpenStats : boolean;
begin
	assign(statfile, fromnts(@cfg.cmpath)+'STATS.CM');
	{$i-} reset(statfile, 1); {$I+}

	if ioresult <> 0 then
	begin
		write('Unable to open STATS.CM.  ');
		pok;
		OpenStats := false;
		exit;
	end;

	blockread(statfile, mainstats, sizeof(_mainstats));

	dosdate(date);
	fillchar(total, sizeof(total), 0);

	OpenStats := true;
end;

procedure CloseStats;
begin
	close(statfile);
end;

function GetOutputFile : boolean;
var
	s : string;
	r : word;
begin
	GetOutputFile := false;

	writeln(#10'Enter the name of the file to output results to, press [ENTER] to output');
	write('to the screen, or [ESC] to quit:'#13#10'  ');

	s := getlinep('', 60, true, false, false, 0, 0, r, #0, false);
	if r = 27 then exit;
	writeln(#10);

	assign(fptr, s);
	{$i-} append(fptr); {$I+}
	if ioresult <> 0 then
	begin
		{$i-} rewrite(fptr); {$I+}
		if ioresult <> 0 then
		begin
			write('Unable to open file for writing.  ');
			pok;
			exit;
		end;
	end;

	GetOutputFile := true;
end;


function GetSystem(var sz : sysznnp; allowwild : boolean) : boolean;
var
	ret		: word;
	s		: string;
	z		: znnp;
begin
	GetSystem := false;

	write(#13#10'Enter node or system name: ');

	s := getlinep('', 26, true, false, false, 0, 0, ret, #0, true);
	if s = '' then
		exit;

	writeln(#13#10);

	if (s = '*') or (strtoznnp(s, z) = false) then
	begin
		z.zone := 0;
		if (not (s[1] in ['A'..'Z','*'])) or
		   (length(s) > 8) or
		   ( (not allowwild) and (s[1] = '*') ) then
		begin
			textcolor(cf.colorchart[ERRCOLOR]);
			write('Invalid node number or system name.  Press any key. ');
			readkey;
			exit;
		end;

		sz.t := DInternet;
		tonts(s, sz.s);
	end
	else
	begin
		if (allowwild = false) and ((z.zone = 65535) or (z.net = 65535) or (z.node = 65535) or (z.point = 65535)) then
		begin
			textcolor(Cf.colorchart[ERRCOLOR]);
			write('Wildcards are not permitted in node number entry. ');
			readkey;
			exit;
		end;

		sz.t := DFidonet;
		sz.z := z;
	end;

	GetSystem := true;
end;


procedure StatsBBS;
begin
	heading('BBS Statistics');

	if not OpenStats then exit;
	if not GetOutputFile then
	begin
		CloseStats;
		exit;
	end;

	d := ConvDTtoT(date);
	d := d - (mainstats.firsttime div 86400);
	if d = 0 then d := 1;

	writeln(fptr, 'BBS Statistics for ',cf.systemname,' generated on ',strdate(date),'.');
	writeln(fptr, '==========================================================================='#10);

	writeln(fptr, '  Messages Imported:        ', mainstats.mimport);
	writeln(fptr, '  Kilobytes Imported:       ', mainstats.kimport);
	writeln(fptr, '  Messages Exported:        ', mainstats.mexport);
	writeln(fptr, '  Kilobytes Exported:       ', mainstats.kexport,#10);

	writeln(fptr, '  Days since installed:     ', d-1);
	writeln(fptr, '  Avg Msgs Imported / Day:  ', mainstats.mimport div d);
	writeln(fptr, '  Avg Kb Imported / Day:    ', mainstats.kimport div d);
	writeln(fptr, '  Avg Msgs Exported / Day:  ', mainstats.mexport div d);
	writeln(fptr, '  Avg Kb Exported / Day:    ', mainstats.kexport div d,#10);

	close(fptr);

	write('Done!  '); pok;
	CloseStats;
end;

procedure StatsNode;
var
	systems : longint;
begin
	heading('Node Statistics');

	systems := 0;

	if not OpenStats then exit;
	if not GetSystem(sz, true) then
	begin
		CloseStats;
		exit;
	end;

	if not GetOutputFile then
	begin
		CloseStats;
		exit;
	end;

	writeln(fptr, 'Statistics for ',cf.systemname,' generated on ',strdate(date),'.');
	writeln(fptr, '==========================================================================='#10);

	seek(statfile, sizeof(_mainstats));
	while not eof(statfile) do
	begin
		blockread(statfile, ns, sizeof(ns));
		if saddrcmp(ns.sz, sz) or ((sz.t = DInternet) and (fromnts(@sz.s) = '*')) then
		begin
			d := ConvDTtoT(date);
			d := d - (ns.first_time div 86400);
			if d = 0 then d := 1;

			writeln(fptr, 'Statistics for system '+sztostr(ns.sz)+':');
			writeln(fptr, '  Messages Received:        ', ns.msg_from);
			writeln(fptr, '  Kilobytes Received:       ', ns.k_from);
			writeln(fptr, '  Duplicates Received:      ', ns.dupes);
			writeln(fptr, '  Messages Sent:            ', ns.msg_to);
			writeln(fptr, '  Kilobytes Sent:           ', ns.k_to);
			writeln(fptr, '  Avg Msgs Received / Day:  ', ns.msg_from div d);
			writeln(fptr, '  Avg Kb Received / Day:    ', ns.k_from div d);
			writeln(fptr, '  Avg Duplicates / Day:     ', ns.dupes div d);
			writeln(fptr, '  Avg Msgs Sent / Day:      ', ns.msg_to div d);
			writeln(fptr, '  Avg Kb Sent / Day:        ', ns.k_to div d,#10);

			with total do
			begin
				inc(msg_from, ns.msg_from);
				inc(k_from, ns.k_from);
				inc(dupes, ns.dupes);
				inc(msg_to, ns.msg_to);
				inc(k_to, ns.k_to);
				inc(systems);
			end;
		end;
	end;

	if systems < 2 then
	begin
		if systems = 0 then
			writeln(fptr, 'No systems matching your criteria were found.'#10);
	end
	else
	begin
		d := ConvDTtoT(date);
		d := d - (mainstats.firsttime div 86400);
		if d = 0 then d := 1;

		writeln(fptr, 'TOTALS:');
		writeln(fptr, '  Messages Received:        ', total.msg_from);
		writeln(fptr, '  Kilobytes Received:       ', total.k_from);
		writeln(fptr, '  Duplicates Received:      ', total.dupes);
		writeln(fptr, '  Messages Sent:            ', total.msg_to);
		writeln(fptr, '  Kilobytes Sent:           ', total.k_to);
		writeln(fptr, '  Avg Msgs Received / Day:  ', total.msg_from div d);
		writeln(fptr, '  Avg Kb Received / Day:    ', total.k_from div d);
		writeln(fptr, '  Avg Duplicates / Day:     ', total.dupes div d);
		writeln(fptr, '  Avg Msgs Sent / Day:      ', total.msg_to div d);
		writeln(fptr, '  Avg Kb Sent / Day:        ', total.k_to div d,#10);
	end;

	close(fptr);

	write('Done!  '); pok;
	CloseStats;
end;


procedure StatsGroup;
var
	systems : longint;
	grpf	: file;
	grp 	: TGroupData;
	s   	: string;
	ret 	: word;
	done	: boolean;
	n		: longint;
	al		: _arealink;
begin
	heading('Group Statistics');

	systems := 0;

	if not OpenStats then exit;

	write(#10'Enter group name: ');
	s := getlinep('', 8, true, false, false, 0, 0, ret, #0, true);
	writeln(#10);
	if s = '' then
	begin
		CloseStats;
		exit;
	end;

	assign(grpf, fromnts(@cfg.cmpath)+'GRPLST.CM');
	{$i-} reset(grpf, 1); {$I+}
	if ioresult <> 0 then
	begin
		write('Unable to open file GRPLST.CM.  ');
		pok;
		CloseStats;
		exit;
	end;

	done := false;
	while not eof(grpf) and not done do
	begin
		blockread(grpf, grp, sizeof(grp));
		if fromnts(@grp.group) = s then
			done := true;
	end;

	close(grpf);

	if done = false then
	begin
		write('Cannot find group ',s,'.  ');
		pok;
		CloseStats;
		exit;
	end;

	assign(grpf, fromnts(@cfg.cmpath)+'GRPMBR.CM');
	{$i-} reset(grpf, 1); {$I+}
	if ioresult <> 0 then
	begin
		write('Unable to open file GRPMBR.CM.  ');
		pok;
		CloseStats;
		exit;
	end;

	if not GetOutputFile then
	begin
		CloseStats;
		close(grpf);
		exit;
	end;

	writeln(fptr, 'Statistics for ',cf.systemname,' generated on ',strdate(date),'.');
	writeln(fptr, '==========================================================================='#10);

	n := grp.members;
	while n > 0 do
	begin
		seek(grpf, n);
		blockread(grpf, al, sizeof(al));

		seek(statfile, sizeof(_mainstats));
		while not eof(statfile) do
		begin
			blockread(statfile, ns, sizeof(ns));
			if saddrcmp(ns.sz, al.sz) then
			begin
				d := ConvDTtoT(date);
				d := d - (ns.first_time div 86400);
				if d = 0 then d := 1;

				writeln(fptr, 'Statistics for system '+sztostr(ns.sz)+':');
				writeln(fptr, '  Messages Received:        ', ns.msg_from);
				writeln(fptr, '  Kilobytes Received:       ', ns.k_from);
				writeln(fptr, '  Duplicates Received:      ', ns.dupes);
				writeln(fptr, '  Messages Sent:            ', ns.msg_to);
				writeln(fptr, '  Kilobytes Sent:           ', ns.k_to);
				writeln(fptr, '  Avg Msgs Received / Day:  ', ns.msg_from div d);
				writeln(fptr, '  Avg Kb Received / Day:    ', ns.k_from div d);
				writeln(fptr, '  Avg Duplicates / Day:     ', ns.dupes div d);
				writeln(fptr, '  Avg Msgs Sent / Day:      ', ns.msg_to div d);
				writeln(fptr, '  Avg Kb Sent / Day:        ', ns.k_to div d,#10);

				with total do
				begin
					inc(msg_from, ns.msg_from);
					inc(k_from, ns.k_from);
					inc(dupes, ns.dupes);
					inc(msg_to, ns.msg_to);
					inc(k_to, ns.k_to);
					inc(systems);
				end;
			end;

			n := al.next;
		end;
	end;

	close(grpf);

	if systems < 2 then
	begin
		if systems = 0 then
			writeln(fptr, 'No systems matching your criteria were found.'#10);
	end
	else
	begin
		d := ConvDTtoT(date);
		d := d - (mainstats.firsttime div 86400);
		if d = 0 then d := 1;

		writeln(fptr, 'TOTALS:');
		writeln(fptr, '  Messages Received:        ', total.msg_from);
		writeln(fptr, '  Kilobytes Received:       ', total.k_from);
		writeln(fptr, '  Duplicates Received:      ', total.dupes);
		writeln(fptr, '  Messages Sent:            ', total.msg_to);
		writeln(fptr, '  Kilobytes Sent:           ', total.k_to);
		writeln(fptr, '  Avg Msgs Received / Day:  ', total.msg_from div d);
		writeln(fptr, '  Avg Kb Received / Day:    ', total.k_from div d);
		writeln(fptr, '  Avg Duplicates / Day:     ', total.dupes div d);
		writeln(fptr, '  Avg Msgs Sent / Day:      ', total.msg_to div d);
		writeln(fptr, '  Avg Kb Sent / Day:        ', total.k_to div d,#10);
	end;

	close(fptr);

	write('Done!  '); pok;
	CloseStats;


end;


end.
