unit areas;

interface

uses crt, dos, cfgdef, defs, strings, putils, data;

type pareaidx = array [0..1000] of _areaidx;


type TArea = object
	areaidx			: ^pareaidx;
    areaidxsize		: longint;
	areafile		: file;

	idx				: longint;

	function  ReadIDX : boolean;
	procedure CloseIDX;

	function GetTag(s : string) : boolean;
	function GetSLArea(s : string) : boolean;
    function GetAreaCfg(os : longint) : boolean;

end;

var	Area : TArea;
	areacfg : _areacfg;


implementation

function TArea.ReadIDX : boolean;
var
 fptr : file;
 lp : longint;
begin
	ReadIDX := false;

	assign(fptr, fromnts(@cfg.cmpath)+'AREAIDX.CM');
    {$i-} reset(fptr, 1); {$I+}
    if ioresult <> 0 then
		exit;

	areaidxsize := filesize(fptr);
	getmem(areaidx, areaidxsize);
	blockread(fptr, areaidx^, areaidxsize);

	close(fptr);

	assign(areafile, fromnts(@cfg.cmpath)+'AREA.CM');
	{$I-} reset(areafile, 1); {$I+}
	if ioresult <> 0 then
		exit;

	ReadIDX := true;
end;

procedure TArea.CloseIDX;
begin
	close(areafile);
	freemem(areaidx, areaidxsize);
end;

function TArea.GetSLArea(s : string) : boolean;
var
	crc  : longint;
	lp   : word;
	w1,w2,c1,c2 : word;

begin
	GetSLArea := true;
	if (s = fromnts(@areacfg.SLArea)) then
		exit;

	crc := $ffffffff;
	for lp := 1 to length(s) do
		crc := crc32(ord(s[lp]), crc);
	c1 := (crc and $ffff0000) shr 16;
	c2 := (crc and $0000ffff);

	lp := areaidx^[0].SLeft;

	while (lp > 0) do
	begin
		if (areaidx^[lp].SLAreaCRC = crc) then
		begin
			idx := lp;
			GetAreaCfg(lp);
	        exit;
		end;

		w1 := (areaidx^[lp].SLAreaCRC and $ffff0000) shr 16;
		w2 := (areaidx^[lp].SLAreaCRC and $0000ffff);

		if (c1 < w1) or ((c1 = w1) and (c2 < w2)) then
			lp := areaidx^[lp].SLeft
		else
			lp := areaidx^[lp].SRight;
	end;

	GetSLArea := false;
	idx := 0;
	exit;
end;


function TArea.GetTag(s : string) : boolean;
var
	crc  : longint;
	lp   : word;
	w1,w2,c1,c2 : word;

begin
	GetTag := true;
	if (areacfg.Tag = s) then
		exit;

	crc := $ffffffff;
	for lp := 1 to length(s) do
		crc := crc32(ord(s[lp]), crc);
	c1 := (crc and $ffff0000) shr 16;
	c2 := (crc and $0000ffff);

	lp := areaidx^[0].TLeft;

	while (lp > 0) do
	begin
		if (areaidx^[lp].TagCRC = crc) then
		begin
			idx := lp;
			GetAreaCfg(lp);
			exit;
		end;

		w1 := (areaidx^[lp].TagCRC and $ffff0000) shr 16;
		w2 := (areaidx^[lp].TagCRC and $0000ffff);

		if (c1 < w1) or ((c1 = w1) and (c2 < w2)) then
			lp := areaidx^[lp].TLeft
		else
			lp := areaidx^[lp].TRight;
	end;

	GetTag := false;
	idx := 0;
	exit;
end;


function TArea.GetAreaCfg(os : longint) : boolean;
begin
	seek(areafile, os*sizeof(_areacfg));

	blockread(areafile, areacfg, sizeof(_areacfg));

	GetAreaCfg := true;
end;


end.