unit dtime;

interface

uses
	filedef, general, dos, utils;

type
	time_t = longint;

	sect = record
		hour, minute, second : byte;
	end;

procedure GetSect(var t : sect);
function  Time : time_t;

procedure ConvTtoDT(d : time_t; var dt : datetype);
function  ConvDTtoT(dt : datetype) : time_t;
function  ConvDTTtoT(dt : datetype; t : sect) : time_t;
procedure ConvTtoS(s : time_t; var t : sect);
function  ConvStoT(t : sect) : time_t;

procedure AddDays(var dt : datetype; n : integer);
procedure AddMonths(var dt : datetype; n : integer);
procedure AddYears(var dt : datetype; n : integer);

function  StrTime(t : sect) : string;
function  StrTimeT(t : time_t) : string;
function  StrDateT(t : time_t) : string;
function  StrDateType(dt : datetype) : string;
function  StrT(t : time_t) : string;

implementation

const
	monlen : array [false..true,1..13] of word
			= ( ( 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365 ),
				( 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 ) );

	monname : array [1..12] of string[5]
			= ( 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul',
				'Aug', 'Sep', 'Oct', 'Nov', 'Dec' );


procedure GetSect(var t : sect);
var
  a, b, c, d : word;
begin
	gettime(a, b, c, d);
	t.hour := a;
	t.minute := b;
	t.second := c;
end;

function Time : time_t;
var
  dt 			: datetype;
  a , b, c, d	: word;
  t				: time_t;
begin
	gettime(a, b, c, d);
	dosdate(dt);

	t := ConvDTtoT(dt);
	t := t + (longint(a) * 3600) + (b * 60) + c;

	Time := t;
end;

function ConvDTTtoT(dt : datetype; t : sect) : time_t;
begin
	ConvDTTtoT := ConvDTtoT(dt) + ConvStoT(t);
end;


function ConvStoT(t : sect) : time_t;
begin
	ConvStoT := t.hour * 3600 + t.minute * 60 + t.second;
end;

procedure ConvTtoS(s : time_t; var t : sect);
begin
	s := s mod 86400;

	t.hour := s div 3600;
	t.minute := (s div 60) mod 60;
	t.second := s mod 60;
end;


procedure ConvTtoDT(d : time_t; var dt : datetype);
var
	done 	: boolean;
	a		: integer;
	leap	: boolean;
begin
	d := d div 86400;

	dt.year := 70;
	done := false;

	while not done do
	begin
		if (dt.year mod 4 = 0) then
			a := 366
		else
			a := 365;

		done := d < a;

		if not done then
		begin
			dec(d, a);
			inc(dt.year);
		end;
	end;

	leap := dt.year mod 4 = 0;

	a := 2;
	while monlen[leap, a] <= d do
		inc(a);

	dt.month := a - 1;
	dt.day := d - monlen[leap, a-1] + 1;
end;


function ConvDTtoT(dt : datetype) : time_t;
var
	leap : boolean;
	d	 : time_T;
begin
	d := (dt.year-70) * 365;
	inc(d, ((dt.year+1-70) div 4));

	leap := dt.year mod 4 = 0;

	inc(d, monlen[leap, dt.month] + dt.day - 1);

	d := d * 86400;
	ConvDTtoT := d;
end;


procedure AddDays(var dt : datetype; n : integer);
var
	d : time_t;
begin
	d := ConvDTtoT(dt);
	inc(d, n * 86400);
	ConvTtoDT(d, dt);
end;

procedure AddMonths(var dt : datetype; n : integer);
var
	a : integer;
begin
	a := dt.month + n;
	inc(dt.year, (a-1) div 12);
	dt.month := (a-1) mod 12 + 1;
end;

procedure AddYears(var dt : datetype; n : integer);
begin
	inc(dt.year, n);
end;

function StrTime(t : sect) : string;
begin
	StrTime := strs2(t.hour) + ':' + strs2(t.minute) + ':' + strs2(t.second);
end;

function StrTimeT(t : time_t) : string;
var
  a : sect;
begin
	ConvTtoS(t, a);
	StrTimeT := StrTime(a);
end;


function StrDateType(dt : datetype) : string;
begin
	StrDateType := strs2(dt.day) + '-' + monname[dt.month] + '-' + strs4(1900+dt.year);
end;

function StrDateT(t : time_t) : string;
var
  dt: datetype;
begin
	ConvTtoDT(t, dt);
	StrDateT := StrDateType(dt);
end;

function StrT(t : time_t) : string;
var
  st : sect;
  dt : datetype;
begin
	ConvTtoDT(t, dt);
	ConvTtoS(t, st);

	StrT := StrDateType(dt) + ' ' + StrTime(st);
end;


end.
