unit slmenu;

interface

uses
	dos, xcrt,
	slexe, output, utils,
	slenter,
	menudef, slmem, filedef, users,
	cm100;

function  RunMenu : boolean;
function  DisplayMenu(name : string) : boolean;
function  ExecuteMenu(name : string) : boolean;


implementation

var
	SystemIn	: integer;

function DispPrompt(m : menutype; pos : integer) : integer;

type
  smtp = ^ScrMenuType;

  cmtp = ^CharMenuType;
  wmtp = ^WordMenuType;
  bmtp = ^BarMenuType;

var
	lp : integer;
	ch : char;
	p  : byte;

	mnu : smtp;
begin
	DispPrompt := 0;

	for lp := 1 to 3 do
	begin
		if m.data.prompt[lp] <> '' then
		begin
			xtextcolor(cf.colorchart[NORMAL]);
			slextwrite(#13#10+m.data.prompt[lp]+' ', true, false);
		end;
	end;


	case m.data.PromptOption[user.help] of
	  SingleChar: mnu := new(cmtp, ct);
	  WordPrompt: mnu := new(wmtp, ct);
	  MenuBar:	  mnu := new(bmtp, ct);
	end;

	mnu^.init;
	for lp := 1 to m.size do
		mnu^.AddItemH(m.items^[lp].name, m.items^[lp].descrip, m.items^[lp].key);

	mnu^.pos := pos;
	mnu^.Display;

	DispPrompt := mnu^.pos;

	dispose(mnu);

	xwrite(#13#10);
end;


procedure DispMenuItems(m : menutype);
var
	s		: string;
	lp		: integer;
	med		: integer;

  function HotIt(s : string; key : char) : string;
  var n : integer;
  begin
	n := pos(key, s);
	if n > 0 then
	begin
		insert(slc(cf.colorchart[SPECIAL]), s, n);
		insert(slc(cf.colorchart[NORMAL]),  s, n+4);
	end;

	HotIt := s;
  end;


begin
	s := slc(cf.colorchart[HEADCOLOR])+'[ '+m.data.title+' ]'+slc(cf.colorchart[NORMAL]);
	while length(s) < 75 do
		s := '-'+s+'-';
	slwriteln(slc(cf.colorchart[NORMAL])+s+#10);

	med := (m.size+1) div 2;

	for lp := 1 to med do
	begin
		s := HotIt(m.items^[lp].name, m.items^[lp].key) + ' ';
		while length(stripcol(s)) < 13 do
			s := s + '.';
		s := s + ' ' + copy(m.items^[lp].descrip, 1, 25) + sp(25-length(m.items^[lp].descrip));

		if lp <= m.size div 2 then
		begin
			s := s + HotIt(m.items^[lp+med].name, m.items^[lp+med].key) + ' ';
			while length(stripcol(s)) < 53 do
				s := s + '.';
			s := s + ' ' + copy(m.items^[lp+med].descrip, 1, 25);
		end;

		slextwrite(s+#13#10, false, false);
	end;

	slwriteln(#10'------------------'+slc(cf.colorchart[HEADCOLOR])+
		'[ Press F1 or ? for Online Help ]'+slc(cf.colorchart[NORMAL])+
		'------------------'+#10);
end;


function ExecuteMenu(name : string) : boolean;
var
	menuptr : ^menudata;
	n		: integer;
begin
	menuptr := @SLData^.SLMenuStack;

	with menuptr^ do
	begin
		n := Menus[Count].Pos;

		if (n = 1) or (Menus[Count].name <> m.name) then
		begin
			if ReadMenu(name, m) = false then
				exit;

			ScanMenu(m, @user.access, user.help, user.pref, SystemIn, cf.ansi);
		end;

		if n > m.size then
		begin
			RemoveMenuFromStack(1);
			exit;
		end;

		inc(Menus[Count].Pos);

		ExecuteCommand(m.items^[n]);
	end;
end;



function DisplayMenu(name : string) : boolean;
var
	s			: string;
	n			: integer;
begin
	DisplayMenu := false;
	if ReadMenu(name, m) = false then
	begin
		xwriteln('Fatal:  Unable to read from menu '+name+'.');
		dec(SLData^.SLMenuStack.Count);
		exit;
	end;

	ScanMenu(m, @user.access, user.help, user.pref, SystemIn, cf.ansi);

	with m.data do
	begin
		xtextcolor(cf.colorchart[NORMAL]);

		if DisplayOption[user.help] and ClearScreen then
			xclrscr;

		if (PromptOption[user.help] = MenuBar) and (DisplayTitle) then
			slwriteln('\WH'+Title+#10);

		xtextcolor(cf.colorchart[NORMAL]);

		if (DisplayOption[user.help] = true) or ForceMenuDisplay then
		begin
			ForceMenuDisplay := false;
			if DisplayFile <> '' then
			begin
				Command160('@'+DisplayFile);
			end
			else
			begin
				if PromptOption[user.help] <> MenuBar then
					DispMenuItems(m);
			end;
		end;

		n := DispPrompt(m, SLData^.SLMenuStack.Menus[SLData^.SLMenuStack.Count].Pos);

		SLData^.SLMenuStack.Menus[SLData^.SLMenuStack.Count].Pos := n;

		ExecuteCommand(m.items^[n]);
	end;
end;


function RunMenu : boolean;
var
	menuptr : ^menudata;
begin
	RunMenu := true;
	menuptr := @SLData^.SLMenuStack;

	if menuptr^.count = 0 then
	begin
		{slwriteln('\WHYou have fallen off the bottom of the stack!');}
		RunMenu := false;
		exit;
	end;

	if menuptr^.menus[menuptr^.count].mode = Display then
		DisplayMenu(menuptr^.menus[menuptr^.count].name)
	else
		ExecuteMenu(menuptr^.menus[menuptr^.count].name);
end;


begin
	SystemIn := 1;
	ForceMenuDisplay := false;
end.
