{$N+,E+}

unit msgs;

interface

uses	dos, crt,
		defs, strings,
		putils, data,
		filedef, sublist, message, general, block, users, post;


type TMsg = object
	p 			: setupptr;
	header      : headertype;
	msg			: msgptr;
	mainstr		: string[60];

	areacfg			: _areacfg;
	areaidx			: _areaidx;

	idxf, areaf	: file;

	constructor MsgOpen;
	destructor  MsgClose;

	function  OpenCMI(subboard, tag : string; var cmif : file) : boolean;
end;

function  CheckAccess(h : headertype; name : string) : boolean;
function  CheckCredits(h : headertype; msg : msgptr; name, origin : string; size : longint) : boolean;

implementation


constructor TMsg.MsgOpen;
begin
	if (not (openfiles([CONFIGF,NODESF])) and (openuserfile)) then
		fail;

	if sublistinit(subboards, '') = false then
		fail;

	assign(areaf, fromnts(@cfg.cmpath)+'AREA.CM');
	{$I-} reset(areaf, 1); {$I+}
	if (ioresult <> 0) then
	begin
		writeln;
		writeln('This program must be executed as part of CatMail.');
		halt(30);
	end;

	assign(idxf, fromnts(@cfg.cmpath)+'AREAIDX.CM');
	{$I-} reset(idxf, 1); {$I+}
	if (ioresult <> 0) then
	begin
		writeln;
		writeln('This program must be executed as part of CatMail.');
		halt(30);
	end;
end;

destructor TMsg.MsgClose;
begin
	closeuserfile;
	closeallfiles;

	close(idxf);
	close(areaf);
end;


function TMsg.OpenCMI(subboard, tag : string; var cmif : file) : boolean;
var
	cmhdr		: _cmhdr;

begin
	OpenCMI := true;
	assign(cmif, fromnts(@cfg.workpath)+subboard+'.CMI');
	{$i-} reset(cmif, 1); {$i+}

	if ioresult <> 0 then
	begin
		rewrite(cmif, 1);
		fillchar(cmhdr, sizeof(cmhdr), 0);
		tonts(subboard, cmhdr.slarea);
		tonts(tag, cmhdr.tag);

		move(cmi_hdr_id, cmhdr.id, 6);

		blockwrite(cmif, cmhdr, sizeof(cmhdr));
		close(cmif);

		{$I-} reset(cmif, 1); {$I+}
		if ioresult <> 0 then
		begin
			write(#13); clreol;
			writeln('   ! Fatal - cannot open ',subboard,'.CMI for reading/writing.');
			log('!', 'Fatal - cannot open '+subboard+'.CMI for reading/writing');
			write(mainstr);
			OpenCMI := false;
			exit;
		end;
	end;
end;


procedure PostMailMessage(filename, name : string; h : headertype; m : msgptr; cost : longint);
var
	header	: headertype;
	msg		: msgptr;

 procedure interpret(var s : string);
 var
	t : string;
	n : integer;
	lp, pp : integer;
 begin
	lp := 1;
	while lp <= length(s) do
	begin
		if s[lp] = '%' then
		begin
			t := copy(s, lp+1, 255);
			n := pos('%', t);
			if n > 0 then
			begin
				delete(t, n, 255);
				upstr(t);
				if t = 'FROM' then
				begin
					if (h.addr.atype = 3) and (m <> Nil) then
						t := strlwr(msgline(m, 1))
					else
						t := h.from;
				end
				else if t = 'TO' then
				begin
					if (h.addr.atype = 4) and (m <> Nil) then
						t := strlwr(msgline(m, 1))
					else
						t := h.touser;
				end
				else if t = 'SUBJECT' then
					t := h.subj
				else if t = 'DATE' then
					t := strdate(h.date)+' '+strs2(h.time.hour)+':'+strs2(h.time.minute)
				else if t = 'COST' then
					t := strs(cost)
				else if t = 'ORIGMSG' then
				begin
					if msg <> Nil then
					begin
						for pp := 1 to m^.msglen do
						begin
							if (pp > 1) or not (h.addr.atype in [3,4]) then
								addline(msg, msgline(m, pp));
						end;
					end;
					t := '';
				end
				else if t = 'ORIGMSG20' then
				begin
					if msg <> Nil then
					begin
						for pp := 1 to m^.msglen do
						begin
							if (pp > 1) or not (h.addr.atype in [3,4]) then
								addline(msg, msgline(m, pp));
							if pp = 20 then
								pp := m^.msglen;
						end;
					end;
					t := '';
				end
				else
					t := '%'+t+'%';

				delete(s, lp, n+1);
				insert(t, s, lp);
			end;
		end;

		inc(lp);
	end;
 end;

 var
	tptr	: text;
	s 		: string;
	subject	: string;

begin
	new(msg);
	clearmsg(msg);

	assign(tptr, fromnts(@cfg.cmpath)+filename);
	{$i-} reset(tptr); {$I+}
	if ioresult <> 0 then
	begin
		addline(msg, 'Unable to open file '+filename+'.  Please alert your SysOp.');
	end
	else
	begin
		readln(tptr, subject);
		while not eof(tptr) do
		begin
			readln(tptr, s);
			interpret(s);
			delete(s, 77, 255);
			addline(msg, s);
		end;

		close(tptr);
	end;

	fillchar(header, sizeof(header), 0);
	with header do
	begin
		from := 'CATMAIL';
		touser := name;
		subj := subject;
		dosdate(date);
		dostime(time);
	end;

	if locksubboard(15, true) then
	begin
		msgpost(msg, header, 0, false, true, true);
		unlocksubboard;
	end;

	disposemsg(msg);
end;



function CheckAccess(h : headertype; name : string) : boolean;
var
	a : attribset;
	lu : usertype;
	luserrec : longint;
	n : byte;

begin
	CheckAccess := false;
	a := [];

	case h.addr.atype of
		1, 2:				{ ignore fidonet messages }
		begin
			CheckAccess := true;
			exit;
		end;

		3: a := icfg.accrecv;
		4: a := icfg.accsend;
	end;

	usearch(name, lu, luserrec);

	if luserrec = 0 then
	begin
		CheckAccess := true;
		exit;
	end;

	for n := 1 to 24 do
	begin
		if (n in a) and (not (n in lu.access.attrib)) then
		begin
			if name = h.from then
				PostMailMessage('CNTSENDA.TXT', name, h, Nil, 0)
			else
				PostMailMessage('CNTRECVA.TXT', name, h, Nil, 0);

			exit;
		end;
	end;

	CheckAccess := true;
end;


function CheckCredits(h : headertype; msg : msgptr; name, origin : string; size : longint) : boolean;
var
	credkb,
	creds		: longint;
	lu 			: usertype;
	luserrec 	: longint;
	n 			: byte;
	fptr		: file;
	tptr		: text;
	vch			: TVolCh;
	t			: o_fi;
	done		: boolean;

	work		: double;

begin
	CheckCredits := true;

	done := false;

	assign(fptr, fromnts(@cfg.cmpath)+'VOLCH.CM');
	{$i-} reset(fptr, 1); {$I+}
	if ioresult <> 0 then
		exit;

	if h.addr.atype in [1,2] then
	begin
		t := DFidonet;
		credkb := cfg.defcredits;
	end
	else
	begin
		t := DInternet;
		credkb := icfg.defcredits;
	end;

	while (not eof(fptr)) and (not done) do
	begin
		blockread(fptr, vch, sizeof(vch));

		if (vch.fi = t) and (WildCardMatch(origin, fromnts(@vch.addr))) then
		begin
			credkb := vch.credits;
			done := true;
		end;
	end;

	close(fptr);

	if credkb = 0 then
		exit;

	usearch(name, lu, luserrec);

	if luserrec = 0 then
		exit;

	work := credkb / 1024 * size;
	creds := round(work);

	if creds > lu.credits then
	begin
		if name = h.from then
			PostMailMessage('CNTSENDC.TXT', name, h, msg, creds)
		else
			PostMailMessage('CNTRECVC.TXT', name, h, msg, creds);

		CheckCredits := false;
		exit;
	end;

	dec(lu.credits, creds);
	writeuser(lu, luserrec);

	if name = h.from then
		PostMailMessage('SENTC.TXT', name, h, msg, creds)
	else
		PostMailMessage('RECDC.TXT', name, h, msg, creds);

	if (fromnts(@cfg.transxlogfile) <> '') then
	begin
		assign(tptr, fromnts(@cfg.transxlogfile));
		{$i-} append(tptr); {$I+}
		if ioresult <> 0 then
		begin
			{$I-} rewrite(tptr); {$I+}
			if ioresult <> 0 then
				exit;
		end;

		writeln(tptr, name, ',', origin, ',', size, ',', credkb, ',', creds);
		close(tptr);
	end;
end;

end.
