#include <stdio.h>
#include <mem.h>
#include "defs.h"
#include "cfgdef.h"
#include "pktdef.h"

#ifndef __CDEFS_H
#define __CDEFS_H

enum RouteType {  RTNothing, NoRoute, RouteTo, FileRoute,
		  HubSend, ForwardFor, ForwardTo,
		  FNormal, FHold, FCrash, FImmediate };

enum DCPkt { DCNone, NetMail, EchoMail, CATFile, FileAttach };




class LList
{
  public:
	void	*data;
	LList 	*next;

  public:
	inline LList(void)
	{
		data = next = NULL;
	}

	inline ~LList(void)
	{
		if (next != NULL)
			delete next;
		delete data;
	}
};


class TLog
{
  public:
	void 	log(char, char *, ...);
	void 	log(o_loglevel, char, char *, ...);
	void 	elog(char *, char, char *, ...);
	void 	error(char *, ...);
	void 	baderror(char *, ...);
};


class TZnnp
{
  public:
	int 	StrToZnnp(char *thing, znnp *z);
	int 	StrToZnnp(char *thing, znnp *z, znnp *old);
	char	*ZnnpToStr(char *thing, znnp *z);
	void 	NodeToStr(char *thing, znnp *z, znnp *lastz);
	void 	PrintAddr(znnp *a);

	int		IsOurAddr(znnp *);
	int 	IsValidAddr(znnp *);

	word 	OurNet(word z);
	word 	OurNode(word z);
	word 	OurPoint(word z);
	word 	OurZone(word z);
	znnp 	*OurZnnp(word z);

	int 	FindAddr(_netmail *, char *, znnp *, znnp *, struct _dupepkt *);
};


class TStats
{
  private:
	FILE		*statptr;

	sysznnp		LastZoneWritten;
	long		LastZonePos;

  public:
	TStats(void);
	~TStats(void);

	_mainstats      MainStats;

	void	ReadStats(sysznnp *z, struct _nodestats *n);
	void 	WriteStats(struct _nodestats *n);
};

class TUtil
{
  public:
	void 	BaseTo36(unsigned b, char *b26);
	word 	GetDate(void);
	ulong 	Crc32(char *);
	ulong 	Crc32(void *, word);
	char	*GetFTSCDate(char *);
};


class TFUtil
{
  public:
	char	LastBracket[30];

	char	*FileOnly(char *);
	int 	MakePath(char *);
	int 	CfgMakePath(char *, char *, int);
	int 	CopyFile(char *, char *);
	int 	RenameToBA(char *);
	int 	RenameToXXX(char *, char *);
	int		IsEnoughHDSpace(char *, long);
	char	*CMPath(char *);
	word 	fngets(char *, word, FILE *);
	char	*frgets(char *, word, FILE *);
};









//  Classes defined in CMARC.CPP  =======================================



class TDPack		// PACK.CM
{
  public:
	znnp	z;
	char	name[15];

	TDPack 	*next;

	TDPack(void);
	~TDPack(void);
};


class TPack
{
  public:
	TDPack	*data;

	TPack(void);
	~TPack(void);

	char *	ArcType(znnp *);
};


class TDomain : public LList
{
  private:
	LList	*domains;

  public:
	TDomain(void);
	~TDomain(void);

	char	*GetDomain(word);
};


class TCPktA
{
  public:
	TCPktA	*next;

	ulong	pkt;
	DCPkt	ptype;

  public:
	TCPktA(ulong, DCPkt);
	TCPktA(void) {}
	~TCPktA();

};

class TCPktB
{
  public:
	TCPktB	*next;

  public:
	char	file[72];

  public:
	TCPktB(char *);
	TCPktB(void) {}
	~TCPktB();
};

class TDCPkt
{
  public:
	znnp	to;
	znnp	unrouted;
	TCPktA	*DataA;
	TCPktB	*DataB;

	RouteType	flavour;
	unsigned	routed : 1;
	unsigned	processed : 1;
	unsigned	closed : 1; 	// has had /0/0/0 placed on end

	TDCPkt	*next;

	TDCPkt(void);
	~TDCPkt(void);
};


class TCPkt
{
  public:
	TDCPkt	*data;
	ulong	PktName;

	TCPkt(void);	// inits PktName
	~TCPkt(void);

	void	Route(void);
	void	Add(znnp *, ulong, DCPkt);
	ulong 	GetPkt(znnp *, DCPkt);
};



class TDPassword
{
  public:
	znnp	z;
	char	pw[16];

	TDPassword	*next;

	TDPassword(void) { next = NULL; }
	~TDPassword(void) { delete next; }
};

class TPassword
{
  private:
	enum o_pw { pPassword, pFroDoor, pDBridge, pSquish };
	void    GetPasswordFile(char *, o_pw);

  public:
	TDPassword	*data;

	char	*Get(znnp *);
	void	Add(znnp *, char *);
	int	Check(znnp *, char *);

	TPassword(void);     // reads in PASSWORD.CM
	~TPassword(void) { delete data; }
};


class TSUtil
{
  public:
	char *	AddSlash(char *);
	char * 	KillRet(char *);
	char *  aword(char *, int, char *);
};

class TSys
{
  public:
	int	CallMailIn;
	int	CallArchiveOut;

	TSys() 	{ CallMailIn = CallArchiveOut = 0; }

	void	bye(byte);
	int	IsAnotherCopyRunning(void);
	void	CopyIsRunning(void);
};









//  Classes defined in CMCPKT.CPP  ======================================



class TZnnpLink
{
  public:
	TZnnpLink	*next;
	znnp		z;

	TZnnpLink(void) { next = NULL; }
	~TZnnpLink(void) { delete next; }

	int	IsInList(znnp *);
};

class TNodeLink
{
  public:
	TNodeLink	*next;
	EMsgStatus	Status;

	sysznnp		sz;

	TNodeLink(void) { next = NULL; }
	~TNodeLink(void) { delete next; }

	int		IsInList(znnp *);
	int		IsInList(char *);
};


/*
class TAreaLink
{
  public:
	TAreaLink	*next;
	znnp		z;
	EMsgStatus	Status;

	TAreaLink(void) { next = NULL; }
	~TAreaLink(void) { delete next; }

	int		IsInList(znnp *);
	EMsgStatus 	GetStatus(znnp *);
};
*/

class TAreaCfg
{
  public:
	char		Tag[40], SLArea[10], Group[10];
	o_dyn	export, import, cstrip, secure, dupe;
	o_uudd	uu;
	byte	isusenet, changed, freeze;

	TNodeLink	*Links;
	char		*Desc;

	TAreaCfg(void);
	~TAreaCfg(void);
};


class TArea
{
  public:
	int		AreaIDXNeedsUpdate;

	word		ndata;
	DAreaIdx	*data;
	TAreaCfg	LastAreaCfg;
	DAreaCfg	LastAreaDCfg;
	long		LastAreaPos;
	FILE		*areafile, *linkfile;

	TArea(void);	// loads it all in
	~TArea(void);

	void		Sort(void);

	void		GenIDX(void);
	void 		SortTreeT(long min, long cur, long max);
	void 		SortTreeS(long min, long cur, long max);

	TAreaCfg *	GetSLArea(char *);
	TAreaCfg *	GetTagArea(char *);

	TAreaCfg * 	GetAreaCfg(long);
	TAreaCfg *  MakeNewArea(char *, sysznnp *, int = 0, char * = NULL, char * = NULL);

	int 		Insert(DAreaCfg *, sysznnp *);
};


class TRegInfo
{
  protected:
	char	regname[25];
	char	regbbs[25];
	word	regnum;
	znnp	regnode;
	byte	keytype;
	long    accession[2], override[2];
	long	expiry;


  public:
	TRegInfo(void);
	~TRegInfo(void);

	word	GetRegNum(void) { return regnum; }
	char	*GetRegName(void) { return regname; }
	char	*GetRegBBS(void) { return regbbs; }
	znnp	*GetRegZnnp(void) { return &regnode; }
	long	GetRegExpiry(void) { return expiry; }

	friend 	TDataBase;
};


class TGenNet
{
  private:
	void 	NextFreeR(char *, char *);

  public:
	int 	SendMsg(struct _netmail *, char *, int, char);
	int 	SendMsg(char *, struct _netmail *, char *, int, char);
	void 	NextFree(char *, int);
};


class TDRoute
{
  public:
	TZnnpLink	*data;
	RouteType	type;

	TDRoute		*next;

	TDRoute(void) { memset(this, 0, sizeof(TDRoute)); }
	~TDRoute(void) { delete data; delete next; }
};


class TRoute
{
  public:
	TDRoute		*RouteData;
	TDRoute		*FlavData;

	TRoute(void) { memset(this, 0, sizeof(TRoute)); }
	~TRoute(void) { delete RouteData;  delete FlavData; }

	int 		ReadCfg(void);
	void 		RouteCPkt(void);
	RouteType	RouteZnnp(znnp *, DCPkt);
	RouteType	GetFlavour(znnp *);

	int	AllowForward(znnp *, RouteType);
};


#endif

#include "e:\source\cm\cmsl.h"
#include "e:\source\cm\gdefs.h"
