#include <alloc.h>
#include <conio.h>
#include <ctype.h>
#include <dir.h>
#include <dos.h>
#include <errno.h>
#include <fcntl.h>
#include <io.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "cfgdef.h"
#include "errors.h"
#include "pktdef.h"
#include "cdefs.h"
#include "gdefs.h"

#include "cmarc.h"
#include "cmdisp.h"


void TArchive::Extract(void)
{
	char 	ext[5], *point;
	int	n;
	struct	ffblk	ff;
	word	nai = 0;


	cprintf("Uncompressing inbound archives...\r\n");

	sprintf(arc, "%s*.*", cfg->filespath);
	n = findfirst(arc, &ff, 0);
	while (!n)
	{
		strcpy(arc, ff.ff_name);
		point = (char *)strchr(arc, '.');
		if (point)
		{
			strcpy(ext, ++point);
			ext[2] = 0;
			if (!stricmp(ext, "MO") ||
			    !stricmp(ext, "TU") ||
			    !stricmp(ext, "WE") ||
			    !stricmp(ext, "TH") ||
			    !stricmp(ext, "FR") ||
			    !stricmp(ext, "SA") ||
			    !stricmp(ext, "SU"))
			{
				nai++;

				sprintf(arc, "%s%s", cfg->filespath, ff.ff_name);
				strupr(arc);

				TDArchive *atype = CompressType(arc);
				if (atype == NULL)
				{
					cprintf("Unknown compression type on packet %s!  Renaming to BAD.\r\n", FUtil.FileOnly(arc));
					Log->log('!', "Unknown compression type on packet %s!", FUtil.FileOnly(arc));
					Log->log('!', "Renaming packet as .BAD", arc);
					FUtil.RenameToBA(arc);
					goto nextfind;
				}

				cprintf("    %s compressed with %s.\r\n", FUtil.FileOnly(arc), atype->Name);
				Log->log(' ', "Received archive %s comp with %s",
					FUtil.FileOnly(arc), atype->Name);
				Interpret(atype->CmdExtract, arc, cfg->workpath, pkt);

				if (cfg->quiet)
					strcat(pkt, " > nul");

				TScreen *ArcScr = new TScreen;
				if (ArcScr != NULL)
					ArcScr->Save();
				n = system(pkt);
				if (ArcScr != NULL)
				{
					ArcScr->Load();
					delete ArcScr;
				}

				if (n == -1)
				{
					switch (n)
					{
					  case ENOMEM:
						cprintf("   ! Not enough memory to run %s.  Packet not extracted.\r\n", atype->Name);
						Log->log('!', "Not enough memory to spawn %s", atype->Name);
						break;
					  default:
						cprintf("   ! Unable to spawn %s.  Packet not extracted.\r\n", atype->Name);
						Log->log('!', "Unable to spawn %s", atype->Name);
						break;
					}
				}

				if (n > 0)
				{
					cprintf("   ! %s returned %d while unarchiving %s.  Renaming as bad.\r\n", atype->Name, n, arc);
					Log->log('!', "%s returned %d while unarchiving %s", atype->Name, n, FUtil.FileOnly(arc));
					Log->log('!', "Renaming %s as .BAD", FUtil.FileOnly(arc));
					FUtil.RenameToBA(arc);
				}
				if (n == 0)
					unlink(arc);
			}
		}
nextfind:
		n = findnext(&ff);
	}

	cprintf("   Uncompressed %d archives.\r\n\n", nai);
}

char *TArchive::GenFDArcName(TDCPkt *CurCPkt, byte *append)
{
	char		*point, ext[10], thing[75];
	TDArcOut 	*ArcInfo = ArcOut->Read(&CurCPkt->to);

	if (ArcInfo == NULL)
		ArcInfo = ArcOut->Write(&CurCPkt->to);

	if (ArcInfo->LastDate != Util.GetDate())
	{
		ArcInfo->LastDate = Util.GetDate();
		ArcInfo->Today = 0;
	}

	time_t t = time(0);
	point = ctime(&t);
	ext[0] = *point++;
	ext[1] = toupper(*point);

	if (ArcInfo->Today == 0)
		ext[2] = '0';
	else
		ext[2] = (ArcInfo->Today-1) > 9 ?
				(ArcInfo->Today-1) + 55 :
				(ArcInfo->Today-1) + 48;
	*append = ext[3] = 0;
	sprintf(thing, "%s%08lX.%s", cfg->packpath, ArcInfo->Pkt, ext);

	if (!access(thing, 0))
	{
		*append = 1;
		return thing;
	}

	ArcInfo->Today++;
	ext[2] = (ArcInfo->Today-1) > 9 ?
			(ArcInfo->Today-1) + 55 :
			(ArcInfo->Today-1) + 48;
	sprintf(thing, "%s%08lX.%s", cfg->packpath, ArcInfo->Pkt, ext);

	return thing;
}


char *TArchive::GenDBArcName(TDCPkt *CurCPkt)
{
	char	thing[75], ext[8];

	strcpy(thing, cfg->packpath);
	strcat(thing, "N-");

	switch (CurCPkt->flavour)
	{
	  case FCrash:
		thing[strlen(thing)-2] = 'C';
		break;

	  case FHold:
		thing[strlen(thing)-2] = 'H';
		break;

	  case FImmediate:
		thing[strlen(thing)-2] = 'I';
		break;
	}

	if (CurCPkt->to.point)
	{
		strcat(thing, "POINT.");
		Util.BaseTo36(CurCPkt->to.point, ext);
		strcat(thing, ext);
	}
	else
	{
		Util.BaseTo36(CurCPkt->to.zone, ext);
		strcat(thing, ext);

		Util.BaseTo36(CurCPkt->to.net, ext);
		strcat(thing, ext);

		Util.BaseTo36(CurCPkt->to.node, ext);
		strcat(thing, ".");
		strcat(thing, ext);
	}

	return thing;
}

int TArchive::Compress(char *arc, char *pkt, TDCPkt *CurCPkt, int append)
{
	TPack Pack;
	TDArchive *Archive = FindName(Pack.ArcType(&CurCPkt->to));

	char	*s = new char[250];

	memassert(s);


	if (Archive == NULL)
	{
		if ( (Archive = arch) == NULL )
		{
			Log->error("No archivers defined inside CMSetup");
			FUtil.RenameToXXX(pkt, "UAP");
			delete s;
			return 1;
		}
	}

	Log->log('+', "Creating archive %s with %s for %s",
		FUtil.FileOnly(arc), Archive->Name,
		Znnp.ZnnpToStr(NULL, &CurCPkt->unrouted));

	Log->log(LDebug, '+', "   containing file %s", FUtil.FileOnly(pkt));

	strcpy(s, FUtil.FileOnly(arc));
	cprintf("   %s : %s with %s for %s [%c]\r\n",
		s, FUtil.FileOnly(pkt), Archive->Name,
		Znnp.ZnnpToStr(NULL, &CurCPkt->to),
		append ? '+' : '|');

	Interpret(Archive->CmdAdd, arc, pkt, s);

	if (cfg->quiet)
		strcat(s, " > nul");

	TScreen *ArcScr = new TScreen;
	if (ArcScr != NULL)
		ArcScr->Save();

	int n = system(s);

	if (ArcScr != NULL)
	{
		if (cfg->quiet)
			ArcScr->Load();
		delete ArcScr;
	}

	if (n == -1)
	{
		switch (n)
		{
		  case ENOMEM:
			Log->error("Not enough memory to run %s.  Packet not compressed.\r\n", Archive->Name);
			break;
		  default:
			Log->error("Unable to spawn %s.  Packet not compressed.\r\n", Archive->Name);
			break;
		}

		FUtil.RenameToXXX(pkt, "UAP");
		delete s;
		return 1;
	}
	else if (n > 0)
	{
		Log->error("%s returned %d while archiving %s.", Archive->Name, n, arc);
		Log->error("Packet not compressed.");

		FUtil.RenameToXXX(pkt, "UAP");

		delete s;
		return 1;
	}
	else if (access(arc, 0))
	{
		Log->error("Compressed file %s not created (unknown reason)", arc);
		Log->error("Packet not compressed.");

		FUtil.RenameToXXX(pkt, "UAP");

		delete s;
		return 1;
	}

	unlink(pkt);

	delete s;
	return 0;
}

int TArchive::Bink(char *arc, TDCPkt *CurCPkt)
{
	TDomain	Domain;
	char	*p, *bpath = new char[80], *s = new char[80];


	strcpy(bpath, cfg->binkpath);
	if ((p = Domain.GetDomain(CurCPkt->to.zone)) == NULL)
		strcat(bpath, "OUTBOUND");
	else
		strcat(bpath, p);

	if (CurCPkt->to.zone != cfg->address[0].zone)
	{
		sprintf(s, ".%03x", CurCPkt->to.zone);
		strcat(bpath, s);
	}

	if (CurCPkt->to.point)
	{
		SUtil.AddSlash(bpath);
		sprintf(s, "%04X%04X.PNT", CurCPkt->to.net, CurCPkt->to.node);
		strcat(bpath, s);
	}
	strupr(bpath);
	SUtil.AddSlash(bpath);

	if (FUtil.MakePath(bpath))
	{
		Log->error("Unable to make directory %s.  PKT not compressed.", bpath);

		FUtil.RenameToXXX(pkt, "UAP");
		delete s;
		delete bpath;
		return 1;
	}


	if (CurCPkt->to.point)
		sprintf(s, "%08X.FLO", CurCPkt->to.point);
	else
		sprintf(s, "%04X%04X.FLO", CurCPkt->to.net, CurCPkt->to.node);

	strcat(bpath, s);

	switch (CurCPkt->flavour)
	{
	  case FCrash:
	  case FImmediate:
		bpath[strlen(bpath)-2] = 'C';
		break;

	  case FHold:
		bpath[strlen(bpath)-2] = 'H';
		break;
	}

	strupr(bpath);

	FILE *fptr = fopen(bpath, "a");
	if (fptr == NULL)
	{
		Log->error("Unable to append to file %s for BinkleyTerm attach", bpath);
		delete bpath;
		delete s;
		return 1;
	}

	switch (cfg->sentaction)
	{
	  case SA_Trunc:
		putc('#', fptr);
		break;
	  case SA_Kill:
		putc('^', fptr);
		break;
	}

	fputs(arc, fptr);
	putc('\n', fptr);

	TCPktB *b = CurCPkt->DataB;
	while (b)
	{
		fprintf(fptr, "%s\n", b->file);
		b = b->next;
	}

	fclose(fptr);
	delete bpath;
	delete s;

	return 0;
}

int TArchive::FrontDoor(char *arc, TDCPkt *CurCPkt)
{
	struct _netmail *net = new _netmail;

	char	*str = new char[80];
	time_t	tm;
	struct	tm	*tnow;


	GenNet.NextFree(str, 1);

	FILE *fptr = fopen(str, "wb");

	memset(net, 0, sizeof(_netmail));
	strcpy(net->fromuser, "ARCmail");
	strcpy(net->touser, "SysOp");
	strupr(arc);
	strcpy(net->subject, arc);

	time(&tm);
	tnow = localtime(&tm);
	strftime(net->datetime, 20, "%d %b %y  %H:%M:%S", tnow);

	net->destnode = CurCPkt->to.node;
	net->orignode = Znnp.OurNode(CurCPkt->to.zone);
	net->orignet = Znnp.OurNet(CurCPkt->to.zone);
	net->destnet = CurCPkt->to.net;
	net->attrib = KillSent | Priv | Local | FileAttached;

	fwrite(net, sizeof(_netmail), 1, fptr);
	delete net;

	fprintf(fptr, "\001INTL %s %s\r",
		Znnp.ZnnpToStr(NULL, &CurCPkt->to),
		Znnp.ZnnpToStr(str, Znnp.OurZnnp(CurCPkt->to.zone)));


	if (CurCPkt->to.point)
		fprintf(fptr, "\001TOPT %d\r", CurCPkt->to.point);

	if (Znnp.OurPoint(CurCPkt->to.zone))
		fprintf(fptr, "\001FMPT %d\r", Znnp.OurPoint(CurCPkt->to.zone));


	switch (cfg->sentaction)
	{
	  case SA_Kill:
		fprintf(fptr, "\001FLAGS KFS\r");
		break;
	  case SA_Trunc:
		fprintf(fptr, "\001FLAGS TFS\r");
		break;
	}

	fclose(fptr);
	delete str;

	return 0;
}

void TArchive::Comment(char *arc, TDCPkt *CurCPkt)
{
	char *str = new char[80];

	sprintf(str, "%s%s", cfg->packpath, cfg->comfile);

	FILE *fptr = fopen(str, "a");

	if (CurCPkt->routed)
		fprintf(fptr, "%s ARCMail for node %s (routed through %s)\n", FUtil.FileOnly(arc), Znnp.ZnnpToStr(NULL, &CurCPkt->unrouted), Znnp.ZnnpToStr(NULL, &CurCPkt->to));
	else
		fprintf(fptr, "%s ARCMail for node %s\n", FUtil.FileOnly(arc), Znnp.ZnnpToStr(NULL, &CurCPkt->to));

	fclose(fptr);
	_chmod(str, 1, FA_HIDDEN|FA_ARCH);

	delete str;
}


int TArchive::Archive(void)
{
	word	npc = 0;

	cprintf("Compressing outbound packets...\r\n");

	Route.RouteCPkt();

	TDCPkt	*CurCPkt = CPkt.data;
	while (CurCPkt)
	{
		if (CurCPkt->processed)
		{
			CurCPkt = CurCPkt->next;
			continue;
		}

		npc++;

		byte append = 1;

		if (cfg->mailertype != M_DBridge)
			strcpy(arc, GenFDArcName(CurCPkt, &append));
		else
			strcpy(arc, GenDBArcName(CurCPkt));


		TCPktA *OutPkt = CurCPkt->DataA;


		int n = 0;
		while (OutPkt)
		{
			sprintf(pkt, "%sOUTPKT\\%08lX.%s", cfg->workpath,
				OutPkt->pkt,
				OutPkt->ptype == CATFile ? "CAT" : "PKT");

			if (!!(n |= Compress(arc, pkt, CurCPkt, append)))
			{
				OutPkt = OutPkt->next;
				continue;
			}

			OutPkt = OutPkt->next;
		}

		if (n)	// if there was an error archiving, bugger off...
		{
			CurCPkt = CurCPkt->next;
			continue;
		}

		if (append)  // if it's only appending, bugger off too :)
		{
			CurCPkt->processed = 1;
			CurCPkt = CurCPkt->next;
			continue;
		}

		switch (cfg->mailertype)
		{
		  case M_Binkley:
		  case M_Portal:
			if (Bink(arc, CurCPkt))
			{
				CurCPkt = CurCPkt->next;
				continue;
			}
			break;

		  case M_FrontDoor:
			FrontDoor(arc, CurCPkt);
			break;
		}

		if (cfg->usecomfile)
			Comment(arc, CurCPkt);


		CurCPkt->processed = 1;
		CurCPkt = CurCPkt->next;
	}

	cprintf("   Compressed %d packets.\r\n\n", npc);

	return OK;
}



TDArchive *TArchive::CompressType(char *file)
{
	TDArchive	*a;
	FILE		*fptr;
	int			lp;
	char		tmp[5];


	if (arch == NULL)
		return NULL;

	fptr = fopen(file, "rb");
	if (fptr == NULL)
		return NULL;

	a = arch;
	do
	{
		char *s = new char[strlen(a->IdString)+2];
		*s = 0;

		if (a->IdLocation < 0)
			fseek(fptr, abs(a->IdLocation), SEEK_END);
		else
			fseek(fptr, a->IdLocation, SEEK_SET);

		for (lp=0; lp<strlen(a->IdString)/2; lp++)
		{
			sprintf(tmp, "%02X", getc(fptr));
			strcat(s, tmp);
		}

		if (!stricmp(s, a->IdString))
		{
			fclose(fptr);
			delete s;
			return a;
		}

		delete s;

		if (a->next == NULL) break;
		a = a->next;
	}
	while (1);

	fclose(fptr);
	return NULL;
}


void TArchive::Interpret(char *str, char *a, char *f, char *buf)
{
	char *point = str;
	char *buff = buf;
	char *p;

	*buff = 0;
	while (*point)
	{
		if (*point == '%')
		{
			if (tolower(*(point+1)) == 'a')
			{
				point++;
				p = a;
				while (*p)
					*buff++ = *p++;
			}
			else if (tolower(*(point+1)) == 'f')
			{
				point++;
				p = f;
				while (*p)
					*buff++ = *p++;
			}
			else
				*buff++ = *point;
		}
		else
			*buff++ = *point;

		point++;
	}

	*buff = 0;
}



TDomain::TDomain(void)
{
	LList	*l;
	FILE	*dptr;

	l = domains = new LList;

	dptr = fopen(FUtil.CMPath("DOMAIN.CM"), "rb");

	while (!feof(dptr))
	{
		l->data = new _domain;
		fread(l->data, sizeof(_domain), 1, dptr);
		l->next = new LList;
		l = l->next;
	}

	fclose(dptr);
}

char *TDomain::GetDomain(word z)
{
	LList	*d;
	_domain	*dd;

	d = domains;

	while (d->data)
	{
		dd = (_domain *)d->data;
		if ( dd->zone == z)
			return (char *)dd->path;
		d = d->next;
	}

	return (char *)NULL;
}

TDomain::~TDomain(void)
{
	delete domains;
}




//  TCPkt class


TDCPkt::TDCPkt(void)
{
	memset(this, 0, sizeof(TDCPkt));
	flavour = FNormal;
}

TDCPkt::~TDCPkt(void)
{
	if (next) delete next;
}


TCPkt::TCPkt(void)
{
	ulong t = time(0);
	data = NULL;
	PktName = 0;

	int lp;
	for (lp=0; lp<32; lp++)
	{
		PktName+= (t & 1) << (31-lp);
		t>>= 1;
	}
}

TCPkt::~TCPkt(void)
{
	if (data) delete data;
}

ulong TCPkt::GetPkt(znnp *z, DCPkt tpkt)
{
	TDCPkt	*l = data;
	TCPktA	*a;

	while (l)
	{
		if (*z == l->to)
		{
			a = l->DataA;
			while (a)
			{
				if (a->ptype == tpkt)
					return a->pkt;
				a = a->next;
			}
		}

		l = l->next;
	}

	Add(z, PktName, tpkt);
	return PktName++;
}

void TCPkt::Add(znnp *to, ulong pkt, DCPkt tpkt)
{
	TDCPkt	*l = data;
	TCPktA	*a;

	while (l)
	{
		if (&l->to == to)
		{
			a = l->DataA;
			if (a != NULL)
			{
				while (a)
				{
					if (a->next == NULL)
						break;
					a = a->next;
				}

				a->next = new TCPktA(pkt, tpkt);
			}
			else
				a = new TCPktA(pkt, tpkt);
		}

		if (l->next == NULL)
			break;
		l = l->next;
	}

	if (data == NULL)
	{
		data = new TDCPkt;
		memcpy(&data->to, to, sosz);
		memcpy(&data->unrouted, to, sosz);
		data->DataA = new TCPktA(pkt, tpkt);
	}
	else
	{
		l->next = new TDCPkt;
		l = l->next;
		memcpy(&l->to, to, sosz);
		memcpy(&l->unrouted, to, sosz);
		l->DataA = new TCPktA(pkt, tpkt);
	}
}



TCPktA::TCPktA(ulong pk, DCPkt tpkt)
{
	memset(this, 0, sizeof(TCPktA));
	pkt = pk;
	ptype = tpkt;
}

TCPktA::~TCPktA()
{
	if (next) delete next;
}


TCPktB::TCPktB(char *fn)
{
	memset(this, 0, sizeof(TCPktB));
	strcpy(file, fn);
}

TCPktB::~TCPktB()
{
	if (next) delete next;
}






//  TArchive Class


TDArchive::TDArchive(void)
{
	memset(this, 0, sizeof(TDArchive));
}

TDArchive::~TDArchive(void)
{
	if (next) delete next;
}


TDArchive *TArchive::FindName(char *n)
{
	TDArchive	*at;
	int		quit = 0;

	at = arch;

	do
	{
		if (!stricmp(n, at->Name))
			return at;

		if (at->next == NULL) quit = 1; else at = at->next;
	}
	while (!quit);

	Log->error("Can't find archiver name %s - using default", n);

	return NULL;
}


TArchive::TArchive(void)
{
	FILE 		*fptr;
	TDArchive 	*a;

	arch = NULL;

	fptr = fopen(FUtil.CMPath("ARCHIVE.CM"), "rb");
	if (fptr != NULL)
	{
		if (!feof(fptr))
		{
			arch = new TDArchive;
			fread(arch, sizeof(TDArchive)-4, 1, fptr); // -4 for the *ptr
			arch->next = NULL;
			a = arch;

			while (!feof(fptr))
			{
				a->next = new TDArchive;
				a = a->next;
				fread(a, sizeof(TDArchive)-4, 1, fptr);
				a->next = NULL;
			}
		}

		fclose(fptr);
	}

	arc = new char[80];
	pkt = new char[80];

	ArcOut = new TArcOut;

	memassert(ArcOut);
}

TArchive::~TArchive(void)
{
	delete arch;
	delete arc;
	delete pkt;

	delete ArcOut;
}





//  TPack Class


TDPack::TDPack(void)
{
	memset(this, 0, sizeof(TPack));
}


TDPack::~TDPack(void)
{
	if (next) delete next;
}


TPack::TPack(void)
{
	FILE 		*fptr;
	TDPack		*p;

	data = NULL;

	fptr = fopen(FUtil.CMPath("PACK.CM"), "rb");
	if (fptr == NULL)
		return;

	if (feof(fptr))
	{
		fclose(fptr);
		return;
	}

	p = data = new TDPack;
	fread(p, sizeof(TDPack)-4, 1, fptr);
	p->next = NULL;

	while (!feof(fptr))
	{
		p->next = new TDPack;
		p = p->next;
		fread(p, sizeof(TDPack)-4, 1, fptr);
		p->next = NULL;
	}

	fclose(fptr);
}

TPack::~TPack(void)
{
	if (data) delete data;
}

char *TPack::ArcType(znnp *z)
{
	TDPack	*pt;

	pt = data;

	do
	{
		if (z->equiv(pt->z))
			return pt->name;

		if (pt->next == NULL)
			return NULL;

		pt = pt->next;
	}
	while (1);
}








//  TArcOut Class

TDArcOut::TDArcOut(void)
{
	memset(this, 0, sizeof(TDArcOut));
}

TDArcOut::~TDArcOut(void)
{
	if (next) delete next;
}


TArcOut::TArcOut(void)
{
	FILE 		*fptr;
	TDArcOut	*p, temp;

	data = NULL;

	fptr = fopen(FUtil.CMPath("ARCOUT.CM"), "rb");
	if (fptr == NULL)
		return;

	if (filelength(fileno(fptr)) == 0)
	{
		fclose(fptr);
		return;
	}

	while (!feof(fptr))
	{
		if (fread(&temp, sizeof(TDArcOut)-4, 1, fptr) == NULL)
			break;

		if (data == NULL)
			p = data = new TDArcOut;
		else
		{
			p->next = new TDArcOut;
			p = p->next;
		}

		memcpy(p, &temp, sizeof(TDArcOut)-4);
	}

	fclose(fptr);
}

TArcOut::~TArcOut(void)
{
	TDArcOut	*p = data;
	FILE		*fptr;

	fptr = fopen(FUtil.CMPath("ARCOUT.CM"), "wb");

	if (data == NULL) { delete data; fclose(fptr); return; }

	do
	{
		fwrite(p, sizeof(TDArcOut)-4, 1, fptr);
		if (p->next == NULL)
			{ delete data; fclose(fptr); return; }
		p = p->next;
	}
	while (1);
}


TDArcOut *TArcOut::Write(znnp *z)
{
	long		name;
	TDArcOut	*p = data;
	int		NameOkay = 0;

	if (!z->point)
	{
		name = ((long)(z->zone % 256) << 24) +
			((long)(z->net % 4096) << 12) +
			((long)(z->node % 4096));
	}
	else
	{
		name = ((long)(z->zone % 256) << 24) +
		       ((long)(z->net % 256) << 16) +
		       ((long)(z->node % 256) << 8) +
		       (z->point % 256);
	}

	while (!NameOkay)
	{
		p = data;
		if (data == NULL)
			break;

		do
		{
			if (p->To == *z)
				break;

			if (p->next == NULL)
			{
				NameOkay = 1;
				name--;
				break;
			}

			p = p->next;
		}
		while (1);

		name++;
	}

	return Write(z, name, 0);
}


TDArcOut *TArcOut::Write(znnp *z, long pkt, word today)
{
	TDArcOut	*p = data;

	if (data == NULL)
		p = data = new TDArcOut;
	else
	{
		while (1)
		{
			if (*z == p->To)
			{
				p->Pkt = pkt;
				p->Today = today;
				p->LastDate = Util.GetDate();
				return p;
			}

			if (p->next == NULL)
				break;

			p = p->next;
		}

		p->next = new TDArcOut;
		p = p->next;
	}

	p->next = NULL;
	memcpy(&p->To, z, sosz);
	p->Pkt = pkt;
	p->Today = today;
	p->LastDate = Util.GetDate();

	return p;
}

TDArcOut *TArcOut::Read(znnp *z)
{
	TDArcOut	*p = data;

	if (data == NULL) return NULL;

	while (1)
	{
		if (p->To == *z)
			return p;

		if (p->next == NULL)
			return NULL;

		p = p->next;
	}
}
