#include <alloc.h>
#include <conio.h>
#include <io.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>

#include <assert.h>

#include "cdefs.h"
#include "cmutils.h"

#define max(x, y)  ((x) > (y) ? (x) : (y))

int TNodeLink::IsInList(znnp *n)
{
	TNodeLink 	*a = this;

	while (a)
	{
		if (a->sz.equiv(*n))
			return Status;
		a = a->next;
	}
	return 0;
}

int TNodeLink::IsInList(char *System)
{
	TNodeLink *a = this;

	while (a)
	{
		if (a->sz.equiv(System))
			return Status;
		a = a->next;
	}
	return 0;
}


int TZnnpLink::IsInList(znnp *n)
{
	TZnnpLink *a;

	if (z.equiv(*n))
		return 1;

	a = next;
	while (a)
	{
		if (a->z.equiv(*n))
			return 1;

		a = a->next;
	}
	return 0;
}

int znnp::operator==(const znnp &r) const
{
	return (zone == r.zone && net == r.net && node == r.node && point == r.point);
}

int	znnp::equiv(const znnp &r) const
{
	return ( (zone  == r.zone  || zone  == 65535 || r.zone  == 65535) &&
			 (net   == r.net   || net   == 65535 || r.net   == 65535) &&
			 (node  == r.node  || node  == 65535 || r.node  == 65535) &&
			 (point == r.point || point == 65535 || r.point == 65535));
}

int sysznnp::operator==(const sysznnp &r) const
{
	if (r.t == DFidoNet)
		return (z == r.z);

	return !stricmp(s, r.s);
}

int sysznnp::operator==(const znnp &r) const
{
	return (t == DInternet) ? 0 : r == z;
}

int sysznnp::operator==(const char *r) const
{
	return (t == DFidoNet) ? 0 : !stricmp(s, r);
}

int	sysznnp::equiv(const sysznnp &r) const
{
	if (r.t == DFidoNet)
		return z.equiv(r.z);

	return WildCardMatch(r.s, s);
}

int sysznnp::equiv(const znnp &r) const
{
	return (t == DInternet) ? 0 : r.equiv(z);
}

int sysznnp::equiv(const char *r) const
{
	return (t == DFidoNet) ? 0 : WildCardMatch(r, s);
}


TArea::TArea(void)
{
	FILE	*idxfile;

	memset(this, 0, sizeof(TArea));

	AreaIDXNeedsUpdate = 1;

	areafile = fopen(FUtil.CMPath("AREA.CM"), "r+b");

	if (areafile == NULL ||
	    filelength(fileno(areafile)) < 2*sizeof(DAreaCfg))
	{
		Log->error("You must set up at least one area in CMSetup before running CM.EXE.");
		Sys.bye(10);
	}

	linkfile = fopen(FUtil.CMPath("AREALINK.CM"), "r+b");

	idxfile = fopen(FUtil.CMPath("AREAIDX.CM"), "rb");
	if (idxfile != NULL)
	{
		struct	stat 	s1, s2;

		fstat(fileno(idxfile), &s1);
		fstat(fileno(areafile), &s2);

		if (s1.st_atime >= s2.st_atime) // if idx older than area
		{
			AreaIDXNeedsUpdate = 0;

			ndata = filelength(fileno(areafile)) / sizeof(DAreaCfg);

			data = (DAreaIdx *)farmalloc((ndata+2) * sizeof(DAreaIdx));
			memassert(data);

			fread(data, sizeof(DAreaIdx), ndata, idxfile);
		}

		fclose(idxfile);
	}
}


TArea::~TArea(void)
{
	fclose(areafile);
	fclose(linkfile);

	farfree(data);
}


struct Dsa
{
	ulong crc, pos;
} *sa;

inline int lt(const void *a, const void *b)
{
	Dsa *aa = (Dsa *)a;
	Dsa *bb = (Dsa *)b;

	return (aa->crc == bb->crc) ? 0 : (aa->crc < bb->crc) ? -1 : 1;
}

void TArea::SortTreeT(long min, long cur, long max)
{
	if (cur > min+1)
	{
		long x = (cur+min)/2;

		data[sa[cur].pos].TLeft = sa[x].pos;
		SortTreeT(min, x, cur);
	}

	if (cur < max-1)
	{
		long x = (cur+max)/2;

		data[sa[cur].pos].TRight = sa[x].pos;
		SortTreeT(cur, x, max);
	}
}


void TArea::SortTreeS(long min, long cur, long max)
{
	if (cur > min+1)
	{
		long x = (cur+min)/2;

		data[sa[cur].pos].SLeft = sa[x].pos;
		SortTreeS(min, x, cur);
	}

	if (cur < max-1)
	{
		long x = (cur+max)/2;

		data[sa[cur].pos].SRight = sa[x].pos;
		SortTreeS(cur, x, max);
	}
}


void TArea::GenIDX(void)
{
	DAreaCfg	a;
	long		lp = 1, ns;

	if (data != NULL)
		farfree(data);

	ndata = filelength(fileno(areafile)) / sizeof(DAreaCfg);
	data = (DAreaIdx *)farcalloc(ndata + 2, sizeof(DAreaIdx));
	sa = (Dsa *)farcalloc(ndata + 2, sizeof(Dsa));

	if (sa == NULL)
	{
		Log->error("Not enough memory to compile area IDX file.");
		Sys.bye(4);
	}

	cputs("      DEBUG: TArea::GenIDX reading AREA.CM\r\n");

//	fseek(areafile, 0L, SEEK_SET);
	fseek(areafile, sizeof(DAreaCfg), SEEK_SET);

	while (fread(&a, sizeof(DAreaCfg), 1, areafile) != NULL)
	{
		memset(&data[lp], 0, sizeof(DAreaIdx));
		memset(&sa[lp], 0, sizeof(Dsa));

		sa[lp].pos = lp;
		sa[lp].crc = data[lp].TagCrc = Util.Crc32(a.Tag);

		if (*a.SLArea)
			data[lp].SLAreaCrc = Util.Crc32(a.SLArea);

		lp++;
	}

	cputs("      DEBUG: Sorting tag records\r\n");
	qsort(&sa[1], ndata-1, sizeof(Dsa), lt);

	cputs("      DEBUG: Building tag tree\r\n");
	data[0].TLeft = data[0].TRight = sa[ndata / 2].pos;
	SortTreeT(0L, ndata / 2, ndata);

	cputs("      DEBUG: Loading SLArea records\r\n");
	for (lp=1, ns=1; lp<ndata; lp++)
	{
		if (data[lp].SLAreaCrc)
		{
			sa[ns].pos = lp;
			sa[ns].crc = data[lp].SLAreaCrc;
			ns++;
		}
	}

	cputs("      DEBUG: Sorting SLArea records\r\n");
	qsort(sa, ns, sizeof(Dsa), lt);

	cputs("      DEBUG: Building SLArea tree\r\n");
	data[0].SLeft = data[0].SRight = sa[ns / 2].pos;
	SortTreeS(0L, ns / 2, ns);

	farfree(sa);


	cputs("      DEBUG: Writing AREAIDX.CM to disk\r\n");
	FILE *fptr = fopen(FUtil.CMPath("AREAIDX.CM"), "wb");
	fwrite(data, sizeof(DAreaIdx), ndata, fptr);
	fclose(fptr);

	cputs("      DEBUG: TArea::GenIDX done!\r\n");
}


TAreaCfg *TArea::GetSLArea(char *slname)
{
	ulong 	c = Util.Crc32(slname);
	long	lp;

	if (!stricmp(LastAreaCfg.SLArea, slname))
		return &LastAreaCfg;

	lp = data[0].SLeft;

	while (lp != 0)
	{
		if (data[lp].SLAreaCrc == c)
		{
			TAreaCfg *ac = GetAreaCfg(lp);

			if (!stricmp(ac->SLArea, slname))
				return ac;
			lp = data[lp].SRight;
		}
		else if (data[lp].SLAreaCrc > c)
			lp = data[lp].SLeft;
		else
			lp = data[lp].SRight;
	}

	return NULL;
}


TAreaCfg *TArea::GetTagArea(char *s)
{
	ulong 	c;
	long	lp;
	char	tag[50];

	strcpy(tag, s);
	strupr(tag);
	c = Util.Crc32(tag);

	if (!stricmp(LastAreaCfg.Tag, tag))
		return &LastAreaCfg;

	lp = data[0].TLeft;

	while (lp != 0)
	{
		if (data[lp].TagCrc == c)
		{
			TAreaCfg *ac = GetAreaCfg(lp);

			if (!stricmp(ac->Tag, tag))
				return ac;
			lp = data[lp].TRight;
		}
		else if (data[lp].TagCrc > c)
			lp = data[lp].TLeft;
		else
			lp = data[lp].TRight;
	}

	return NULL;
}


int TArea::Insert(DAreaCfg *a, sysznnp *link)
{
	ulong	tc, sc, lp, last;


	tc = Util.Crc32(strupr(a->Tag));
	sc = Util.Crc32(a->SLArea);

	last = 0;
	lp = data[0].TLeft;

	while (lp != 0)
	{
		last = lp;

		if (data[lp].TagCrc == tc)
			return 1;

		else if (data[lp].TagCrc > tc)
			lp = data[lp].TLeft;
		else
			lp = data[lp].TRight;
	}

	if (data[last].TagCrc > tc)
		data[last].TLeft = ndata;
	else
		data[last].TRight = ndata;


	if (*a->SLArea)
	{
		lp = data[0].SLeft;
		last = 0;

		while (lp != 0)
		{
			last = lp;

			if (data[lp].SLAreaCrc > sc)
				lp = data[lp].SLeft;
			else
				lp = data[lp].SRight;
		}

		if (data[last].SLAreaCrc > sc)
			data[last].SLeft = ndata;
		else
			data[last].SRight = ndata;
	}

	a->Links = a->Deleted = 0;

	if (link != NULL)
	{
		fseek(linkfile, 0L, SEEK_END);
		a->Links = ftell(linkfile);

		DLink 	l;
		memset(&l, 0, sizeof(l));
		l.Status = MSActive;
		memcpy(&l.sz, link, sizeof(sysznnp));

		fwrite(&l, sizeof(l), 1, linkfile);
	}

	ndata++;
	data = (DAreaIdx *)farrealloc(data, (ndata+2) * sizeof(DAreaIdx));
	if (data == NULL)
	{
		Log->error("Unable to reallocate data block, out of memory.");
		return 1;
	}

	fseek(areafile, 0L, SEEK_END);
	fwrite(a, sizeof(DAreaCfg), 1, areafile);

	memset(&data[ndata-1], 0, sizeof(DAreaIdx));
	data[ndata-1].TagCrc = tc;
	data[ndata-1].SLAreaCrc = sc;

	FILE *fptr = fopen(FUtil.CMPath("AREAIDX.CM"), "wb");
	fwrite(data, sizeof(DAreaIdx), ndata, fptr);
	fclose(fptr);

	return 0;
}





TAreaCfg *TArea::GetAreaCfg(long pos)
{
	DAreaCfg	ac;
	TNodeLink	*zp;
	DLink		a;

	if (pos != 0xffffffffL)
	{
		if (Area->LastAreaPos == pos)
			return &LastAreaCfg;

		fseek(areafile, pos * sizeof(DAreaCfg), SEEK_SET);
	}

	LastAreaPos = pos;
	memset(&Area->LastAreaCfg, 0, sizeof(TAreaCfg)-8);

	if (fread(&ac, sizeof(DAreaCfg), 1, areafile) == NULL)
	{
		LastAreaPos = -1;
		return NULL;
	}

	strcpy(Area->LastAreaCfg.Tag, ac.Tag);
	strcpy(Area->LastAreaCfg.SLArea, ac.SLArea);
	strcpy(Area->LastAreaCfg.Group, ac.Group);

	Area->LastAreaCfg.export = ac.export;
	Area->LastAreaCfg.import = ac.import;
	Area->LastAreaCfg.cstrip = ac.cstrip;
	Area->LastAreaCfg.secure = ac.secure;
	Area->LastAreaCfg.dupe   = ac.dupe;
	Area->LastAreaCfg.uu     = ac.uu;

	Area->LastAreaCfg.isusenet = ac.isusenet;
	Area->LastAreaCfg.changed  = ac.changed;
	Area->LastAreaCfg.freeze   = ac.freeze;

	memcpy(&Area->LastAreaDCfg, &ac, sizeof(DAreaCfg));

	if (LastAreaCfg.Links != NULL)
		delete LastAreaCfg.Links;

	LastAreaCfg.Links = NULL;

	if (ac.Links != 0)
	{
		fseek(linkfile, ac.Links, SEEK_SET);

		while (fread(&a, sizeof(DLink), 1, linkfile) != NULL)
		{
			if (a.Status == MSActive)
			{
				if (LastAreaCfg.Links == NULL)
					LastAreaCfg.Links = zp = new TNodeLink;
				else
				{
					zp->next = new TNodeLink;
					zp = zp->next;
				}
				zp->Status = a.Status;

				memcpy(&zp->sz, &a.sz, sizeof(sysznnp));
			}

			if (a.Next == 0)
				break;

			fseek(linkfile, a.Next, SEEK_SET);
		}
	}

	return &LastAreaCfg;
}


int groupsort(const void *a, const void *b)
{
	DAreaCfg *aa, *bb;

	aa = (DAreaCfg *)a;
	bb = (DAreaCfg *)b;

	return (stricmp(aa->Group, bb->Group));
}

int tagsort(const void *a, const void *b)
{
	DAreaCfg *aa, *bb;

	aa = (DAreaCfg *)a;
	bb = (DAreaCfg *)b;

	return (stricmp(aa->Tag, bb->Tag));
}

void TArea::Sort(void)
{
	if (coreleft() < filelength(fileno(areafile))+2048)
	{
		Log->error("Not enough memory to sort area file.");
		return;
	}

	cprintf("  Debug : Reading... ");

	int recs = filelength(fileno(areafile)) / sizeof(DAreaCfg);

	DAreaCfg *buf = new DAreaCfg[recs];

	recs = fread(buf, sizeof(DAreaCfg), recs, areafile);

	cprintf("sorting(1)... ");
	qsort(buf, recs, sizeof(DAreaCfg), groupsort);

	cprintf("sorting(2)... ");

	char	l[15];
	long	s, lp;

	strcpy(l, buf[0].Group);
	s = 0;
	for (lp=1; lp<recs; lp++)
	{
		if (stricmp(l, buf[lp].Group))
		{
			if (lp-s > 1)
				qsort(&buf[s], lp-s, sizeof(DAreaCfg), tagsort);
			strcpy(l, buf[lp].Group);
			s = lp;
		}
	}

	if (lp-s > 1)
		qsort(&buf[s], lp-s, sizeof(DAreaCfg), tagsort);

	cprintf("writing... ");

	fseek(areafile, 0L, SEEK_SET);
	fwrite(buf, sizeof(DAreaCfg), recs, areafile);

	cprintf("done!\r\n");

	delete buf;
}


TAreaCfg *TArea::MakeNewArea(char *tag, sysznnp *from, int isusenet, char *slarea, char *group)
{
	_newmsg		nm;
	int			fl = 0;

	if (from != NULL)
	{
		FILE *fptr = fopen(FUtil.CMPath("NEWAREA.CM"), "rb");

		while (fread(&nm, sizeof(nm), 1, fptr) != NULL)
		{
			if (from->equiv(nm.from))
			{
				fl = 1;
				break;
			}
		}

		fclose(fptr);

		if (!fl || !nm.autocreate)
			return NULL;
	}

	DAreaCfg 	da;

	memset(&da, 0, sizeof(da));

	strcpy(da.Tag, tag);

	if (group != NULL)
		strcpy(da.Group, group);
	else if (from != NULL)
		strcpy(da.Group, nm.group);

	if (slarea != NULL)
		strcpy(da.SLArea, slarea);

	da.export = da.import = Yes;
	da.isusenet = !!isusenet;
	da.freeze = 1;

	if (from != NULL && nm.autoname && slarea == NULL)
	{
		if (Searchlight->MakeUniqueSLName(tag, da.SLArea, nm.nameprefix) == 0)
			*da.SLArea = 0;
	}

	Insert(&da, from);

	return GetTagArea(tag);
}


TAreaCfg::TAreaCfg(void)
{
	memset(this, 0, sizeof(TAreaCfg));
	Desc = new char[80];
	memassert(Desc);
}

TAreaCfg::~TAreaCfg(void)
{
	delete Links;
	delete Desc;
}

