#include <alloc.h>
#include <conio.h>
#include <ctype.h>
#include <dir.h>
#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cdefs.h"
#include "errors.h"
#include "swap\spawno.h"
#include "cmcfg.h"
#include "id.h"


void ReadConfiguration(void)
{
	char *path = new char[120], *p;

	strcpy(path, "CFG.CM");

	p = getenv("CM");
	if (p != NULL)
	{
		strcpy(path, p);
		SUtil.AddSlash(path);
		strcat(path, "CFG.CM");
	}

	// use CFG.CM unless CM environment variable is set.

	if (access(path, 0))
	{
		puts("\nCatMail is unable to locate CFG.CM, the main configuration file.  If you\n" \
			 "haven't run CatMail before, run CMSETUP.EXE now to create the data files.\n" \
			 "If you have, either change to your CatMail directory or set your CM\n" \
			 "environment variable to point to the directory that CatMail is in.\n\n" \
			 "Remember, the CM environment variable takes precedence over the CM.CFG\n" \
			 "in the current directory.  For more help, see the manual.");
		Sys.bye(6);
	}

	FILE *fptr = fopen(path, "rb");
	if (fptr == NULL)
	{
		puts("\nUnable to open the CatMail CFG.CM file - it may be locked.  Please try to\n" \
		     "resolve this problem by rebooting your computer.  Refer to the manual for\n" \
		     "further help.");
		Sys.bye(6);
	}

	if (filelength(fileno(fptr)) != sizeof(_cfg))
	{
		puts("\nYour CFG.CM file is an incorrect length.  This may indicate it is corrupt\n" \
		     "or that you are running an old version.  Refer to the manual for help.");
		Sys.bye(6);
	}

	cfg = new _cfg;
	memassert(cfg);

	fread(cfg, sizeof(_cfg), 1, fptr);

	if (memcmp(cfg->id, CFG_CM_ID, strlen(CFG_CM_ID)))
	{
		puts("\nYour version of CFG.CM is obsolete.  Run CMSETUP to correct.");
		Sys.bye(6);
	}

	fclose(fptr);
	delete path;
}



TPassword::TPassword(void)
{
	memset(this, 0, sizeof(TPassword));
	this->data = NULL;

	struct _password_cm
	{
		znnp	z;
		o_pw	tipe;
		char	pw[31];
	} PasswordCM;

	FILE *fptr = fopen("PASSWORD.CM", "rb");
	if (fptr == NULL)
		return;

	while (fread(&PasswordCM, sizeof(_password_cm), 1, fptr) != NULL)
	{
		if (PasswordCM.tipe == pPassword)
			Add(&PasswordCM.z, PasswordCM.pw);
		else
			GetPasswordFile(PasswordCM.pw, PasswordCM.tipe);
	}

	fclose(fptr);
}

int TPassword::Check(znnp *z, char *pw)
{
	TDPassword	*p = data;

	while (p)
	{
		if (*z == p->z)
			return !stricmp(p->pw, pw);

		p = p->next;
	}

	return 0;
}

void TPassword::Add(znnp *z, char *pw)
{
	if (data == NULL)
	{
		data = new TDPassword;
		memcpy(&data->z, z, sosz);
		strcpy(data->pw, pw);
		data->next = NULL;
		return;
	}

	TDPassword	*p = data;

	while (p->next != NULL)
		p = p->next;

	p->next = new TDPassword;
	p = p->next;
	memcpy(&p->z, z, sosz);
	strcpy(p->pw, pw);

	return;
}


char *TPassword::Get(znnp *z)
{
	TDPassword	*p = data;

	while (p)
	{
		if (p->z == *z)
			return p->pw;
		p = p->next;
	}

	return NULL;
}

void TPassword::GetPasswordFile(char *fn, o_pw tipe)
{
	FILE	*fptr;
	struct _fdpw
	{
		znnp	node;
		char 	pw[9];
		char    dunno;
	} fd;

	char	*thing, *node, *pw;
	znnp	dum;


	thing = new char[120];  node = new char[30];  pw = new char[30];
	memassert(pw);

	fptr = fopen(fn, "rb");
	if (fptr == NULL)
	{
		cprintf("  ! Unable to open password data file %s.\r\n", fn);
		cprintf("      This filename has been specified in your password list as a data file\r\n" \
			"      that contains passwords.  Please check your settings.\r\n");
		Log->log('!', "Unable to open password data file %s", fn);
		Sys.bye(12);
	}

	switch (tipe)
	{
	  case pFroDoor:
		while (fread(&fd, sizeof(fd), 1, fptr))
		{
			if (!fd.node.zone)
			{
				cprintf("  ! FrontDoor password file %s is incorrect in format.\r\n", fn);
				Log->log('!', "Invalid password file %s", fn);
				Sys.bye(12);
			}

			Add(&fd.node, fd.pw);
		}
		break;

	  case pDBridge:
		while (fgets(thing, 120, fptr))
		{
			sscanf(thing, "%s %s", node, pw);
			Znnp.StrToZnnp(node, &dum);
			if (!dum.zone)
			{
				cprintf("  ! DBridge-type password file %s is incorrect in format.\r\n", fn);
				cprintf("      It must ONLY contain lines of the format \"[node] [pw]\".\r\n");
				Log->log('!', "Invalid password file %s", fn);
				Sys.bye(12);
			}

			Add(&dum, pw);
		}
		break;

	  case pSquish:
		while (fgets(thing, 120, fptr))
		{
			char *point = thing;
			while (isspace(*point)) point++;
			if (!strnicmp(point, "password", 8))
			{
				point+= 8;
				sscanf(point, "%s %s", node, pw);
				Znnp.StrToZnnp(node, &dum);
				if (!dum.zone)
				{
					cprintf("  ! Squish-type password file %s is incorrect in format.\r\n", fn);
					cprintf("      It must contain lines of the format \"Password [node] [pw]\".\r\n");
					Log->log('!', "Invalid password file %s", fn);
					Sys.bye(12);
				}

				Add(&dum, pw);
			}
		}
		break;
	}

	fclose(fptr);

	delete thing;  delete node;  delete pw;
}
