#include <stdio.h>
#include <string.h>
#include <io.h>
#include <stdarg.h>

#include "cdefs.h"
#include "cmcmi.h"
#include "id.h"


TCmi::TCmi(void)
{
	memset(this, 0, sizeof(TCmi));
}

TCmi::TCmi(sysznnp *orig, char *tag, char *slarea)
{
	memset(this, 0, sizeof(TCmi));
	OpenCMI(orig, tag, slarea);
}

TCmi::~TCmi(void)
{
	if (IsFileOpen)
		fclose(cmif);
}


int TCmi::OpenCMI(sysznnp *orig, char *tag, char *slarea = NULL)
{
	char	s[80], n[12];

	if (*tag == 0 || tag == NULL)
		return 1;

	if (*slarea == 0 || slarea == NULL)
		sprintf(n, "%lX", Util.Crc32(tag));
	else
		strcpy(n, slarea);

	sprintf(s, "%s%s.CMI", cfg->workpath, n);

	if (access(s, 0))
	{
		_cmhdr h;

		cmif = fopen(s, "w+b");
		memset(&h, 0, sizeof(h));
		strcpy(h.id, CMI_HDR_ID);
		strcpy(h.tag, tag);
		memcpy(&h.Origin, orig, sizeof(sysznnp));
		if (*slarea != 0 && slarea != NULL)
			strcpy(h.slarea, slarea);
		fwrite(&h, sizeof(h), 1, cmif);
	}
	else
	{
		cmif = fopen(s, "r+b");
		fseek(cmif, 0L, SEEK_END);
	}

	if (cmif == NULL)
		return 1;

	IsFileOpen = 1;
	return 0;
}

int TCmi::CloseCMI(void)
{
	if (!IsFileOpen)
		return 1;

	fclose(cmif);
	IsFileOpen = 0;
	return 0;
}

int TCmi::InitMessage(void)
{
	if (MsgInited == MSInitialised)
		return 1;

	MsgInited = MSInitialised;
	BytesWritten = SizeIsSet = 0;
	MsgHdrAt = -1;

	return 0;
}

int TCmi::FillMessageHeader(char *dt, char *frm, char *ifrm, char *to, char *ito, char *subj, ulong size, int isusenet, char *orig_i, ulong hdr)
{
	if (MsgInited != MSInitialised)
		return 2;

	char	MustMoveBack = 0;

	if (MsgHdrAt == -1)
		MsgHdrAt = ftell(cmif);
	else
	{
		MustMoveBack = 1;
		fseek(cmif, MsgHdrAt, SEEK_SET);
	}

	memset(&mh, 0, sizeof(_cmmsg));

	if (dt == NULL)
		Util.GetFTSCDate(mh.datetime);
	else
		strcpy(mh.datetime, dt);

	strcpy(mh.fromuser, frm);
	strcpy(mh.frominet, ifrm);
	strcpy(mh.touser, to);
	strcpy(mh.toinet, ito);
	strcpy(mh.subject, subj);
	strcpy(mh.id, CMI_MSG_ID);

	if (orig_i != NULL)
		strcpy(mh.orig_i, orig_i);

	mh.usenet = !!isusenet;
	mh.ihdrsize = hdr;

	if (size != -1)
	{
		mh.size = size + 1;
		SizeIsSet = size;
	}
	else
		SizeIsSet = 0;

	fwrite(&mh, sizeof(mh), 1, cmif);

	if (MustMoveBack)
		fseek(cmif, 0L, SEEK_END);

	return 0;
}


int TCmi::MakeMessage(FILE *fptr, int UntilValue)
{
	char	s[120], *p, *pp, quit = 0;
	int	n;

	if (MsgInited != MSInitialised)
		return 2;

	if (MsgHdrAt == -1)
		return 3;

	n = 120;
	while (!quit)
	{
		if (SizeIsSet)
		{
			if (BytesWritten >= SizeIsSet)
				break;

			n = (SizeIsSet-BytesWritten) > 120 ? 120 : (SizeIsSet-BytesWritten);
			n++;
		}

		if (fgets(s, n, fptr) == NULL)
			break;

		switch (cr)
		{
		  case CR0d:	break;

		  case CR0d0a:
			p = pp = s;
			while (*p)
			{
				if (*p == 0x0a)
					p++;
				else
					*p++ = *pp++;
			}
			*p = 0;
			break;

		  case CR0a:
			if (s[strlen(s)-1] == '\n')
				s[strlen(s)-1] = '\r';
			break;
		}


		if ( (p = strchr(s, UntilValue)) != NULL )
		{
			*p = 0;
			quit = 1;
		}

		fwrite(s, 1, strlen(s), cmif);
		BytesWritten+= strlen(s);
	}

	return 0;
}
/*
int TCmi::MakeMessage(char *format, ...)
{
	va_list	ag;

	if (MsgInited != MSInitialised)
		return 2;

	if (MsgHdrAt == -1)
		return 3;

	va_start(ag, format);

	BytesWritten+= vfprintf(cmif, format, ag);

	va_end(ag);

	return 0;
}
*/
int TCmi::MakeMessage(char *str)
{
	if (MsgInited != MSInitialised)
		return 2;

	if (MsgHdrAt == -1)
		return 3;

	fputs(str, cmif);
	BytesWritten+= strlen(str);

	return 0;
}


long TCmi::SaveMessage(void)
{
	BytesWritten++;

	fputc(0, cmif);

	if (!SizeIsSet)
	{
		fseek(cmif, MsgHdrAt, SEEK_SET);
		mh.size = BytesWritten;
		fwrite(&mh, sizeof(mh), 1, cmif);
		fseek(cmif, 0L, SEEK_END);
	}

	MsgInited = MSDone;

	return mh.size;
}
