#include <alloc.h>
#include <dir.h>
#include <dos.h>
#include <io.h>
#include <fcntl.h>
#include <conio.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "defs.h"
#include "cdefs.h"
#include "cmdline.h"
#include "ver.h"

#include "cmdisp.h"

class TDLine
{
  public:
	char 		*line;
	TDLine   	*next;

	TDLine() { memset(this, 0, sizeof(TDLine)); line = new char[80]; }
	TDLine(char *s) { memset(this, 0, sizeof(TDLine)); line = new char[strlen(s)+1]; strcpy(line, s); }
	~TDLine() { delete line; delete next; }
};


class TLine
{
	TDLine	*data, *last;
	word	NumLines;

  public:
	TLine() { memset(this, 0, sizeof(TLine)); }
	~TLine() { delete data; }

	void 	AddLine(char *);
	char 	*GetLine(int);

	int 	GetNumLines(void) { return NumLines; }
};


void TLine::AddLine(char *s)
{
	NumLines++;

	if (data == NULL)
	{
		data = last = new TDLine(s);
		return;
	}

	last->next = new TDLine(s);
	last = last->next;
}

char *TLine::GetLine(int n)
{
	if (data == NULL) return NULL;

	TDLine *cur = data;
	while (cur != NULL && n--)
		cur = cur->next;
	if (cur == NULL) return NULL;

	return cur->line;
}



TScreen::TScreen(void)
{
	scrptr = (word far *) (GetVideoMode() ? 0xb0000000 : 0xb8000000);
	EscapeOn = 0;
}

void TScreen::DPuts(word x, word y, void *thingw, word attrib)
{
	unsigned char *thing;
	int offset;
	int lp;

	thing = (unsigned char *) thingw;
	offset = (y * 80) + x;
	for (lp=0; lp<strlen((const char *)thing); lp++)
		*(scrptr + offset + lp) = ((thing[lp] == 255) ? 32 : thing[lp]) | attrib;
}

inline void TScreen::CDPuts(word y, void *s, word t)
{
	DPuts((80-strlen((char *)s))/2, y, s, t);
}

void TScreen::Setup(void)
{
	int	lp, pp;
//	char	thing[120];
	int	mono = GetVideoMode();


	for (lp=0; lp<2000; lp++)
		*(scrptr + lp) = 32 | 0x0700;

	for (lp=0; lp<80*4; lp++)
		*(scrptr + lp) = 32 | (mono?0x7000:0x1700);

	CDPuts(0, "CatMail v"_ver";  The Searchlight Mail Processor.", mono?0x7000:0x1f00);
	CDPuts(1, "Copyright (C)1994-2000 LastByte Software.", mono?0x7000:0x1300);
	//CDPuts(3, thing, mono?0x7000:0x1e00);

	DPuts(1, 0, " _,,/|", mono?0x7000:0x1b00);
	DPuts(1, 1, " \\o o'", mono?0x7000:0x1b00);
	DPuts(1, 2, " =_~_=", mono?0x7000:0x1b00);

	DPuts(72, 0, " |\\,,_", mono?0x7000:0x1b00);
	DPuts(72, 1, " `o o/", mono?0x7000:0x1b00);
	DPuts(72, 2, " =_~_=", mono?0x7000:0x1b00);

	for (lp=0; lp<4; lp++)
		*(scrptr + lp * 80 + 9) = *(scrptr + lp * 80 + 70) = 179 | (mono?0x7000:0x1000);

	for (lp=0; lp<9; lp++)
		*(scrptr + 3 * 80 + lp) = *(scrptr + 3 * 80 + lp + 71) = 32 | 0x0000;

	*(scrptr + 3 * 80 + 9) = 222 | 0x0100;
	*(scrptr + 3 * 80 + 70) = 221 | 0x0100;


	textbackground(BLACK);
	textcolor(LIGHTGRAY);
	window(1, 6, 80, 25);
	clrscr();
}




void TScreen::Help(void)
{
	struct	_cmb	cmb;
	struct	ffblk	f;
	char	ch;
	char	s[80];

	TLine	*t = new TLine;


	t->AddLine("Help for CatMail v"_ver";  The Searchlight Mail Processor.");
	t->AddLine("");
	t->AddLine("Press the [UP] and [DOWN] keys to scroll through help, [ESC] to quit.");
	t->AddLine("");
	t->AddLine("");
	t->AddLine("The general format for the CatMail command line is:");
	t->AddLine("");
	t->AddLine("      CM.EXE [command] [command] [command] [...]");
	t->AddLine("");
	t->AddLine("where [command] is one or more of the following:");


	int lp = 0;

	while (def[lp].ret)
	{
		if ((stricmp(def[lp].cmd, "pack") ||
			cfg->mailertype == M_Binkley ||
			cfg->mailertype == M_Portal) && stricmp(def[lp].cmd, "pok"))
		{
			sprintf(s, "    %-10s %s", def[lp].cmd, def[lp].desc);
			t->AddLine(s);
		}
		lp++;
	}

	int n = findfirst("*.CMB", &f, 0);
	while (!n)
	{
		FILE	*fptr;

		fptr = fopen(f.ff_name, "rb");
		fread(&cmb, sizeof(cmb), 1, fptr);
		fclose(fptr);

		if (stricmp(cmb.id, "CMB1"))
		{
			n = findnext(&f);
			continue;
		}

		t->AddLine("");
		sprintf(s, " * %s; %s", cmb.progname, cmb.progdesc);
		t->AddLine(s);

		lp = 0;
		while (*cmb.cmd[lp].cmd)
		{
			sprintf(s, "    %-10s %s", cmb.cmd[lp].cmd, cmb.cmd[lp].desc);
			t->AddLine(s);
			lp++;
		}

		n = findnext(&f);
	}


	word nl = t->GetNumLines();

	clrscr();
	for (lp=0; lp<=((nl<18)?nl:18); lp++)
	{
		cputs(t->GetLine(lp));
		cputs("\r\n");
	}

	if (nl < 19)
	{
		gotoxy(1, 18);
		Sys.bye(3);
	}

	word	top = 0;

	do
	{
		long tm = time(0) + 30;
		while (!kbhit() && time(0) < tm);

		if (time(0) >= tm)
			break;

		ch = getch();
		if (ch == 0)
		{
			ch = getch();
			if (ch == 80 && top < nl - 16)
			{
				movetext(1,7,80,24, 1,6);
				top++;
				gotoxy(1, 19);
				clreol();
				if (nl >= top+18)
					cputs(t->GetLine(top+18));
			}
			if (ch == 72 && top)
			{
				movetext(1,6,80,23, 1,7);
				top--;

				gotoxy(1, 1);
				clreol();

				if (nl > top)
					cputs(t->GetLine(top));
			}
		}
	}
	while (ch != 27);

	gotoxy(1, 18);
	Sys.bye(3);
}



void TScreen::CMInfo(void)
{
	cprintf("You are running version "_ver" of CatMail.\r\n");
	cprintf("It was compiled under Borland C++ v3.0 on "__DATE__" at "__TIME__".\r\n\n");

	cprintf("There are currently %ld bytes of memory free.\r\n", coreleft());
}


int TScreen::GetVideoMode(void)
{
	union	REGS	regs;

	regs.h.ah = 0x0f;
	int86(0x10, &regs, &regs);
	return (regs.h.al == 7) ? 1 : 0;
}

void TScreen::Save(void)
{
	scrbuf = new word[2000];
	OldX = wherex();
	OldY = wherey();

	if (scrbuf != NULL)
		memcpy((const void *)scrbuf, (const void *)scrptr, 4000);
}

void TScreen::Load(void)
{
	if (scrbuf != NULL)
	{
		memcpy((const void *)scrptr, (const void *)scrbuf, 4000);
		delete scrbuf;
	}
	gotoxy(OldX, OldY);
}

void TScreen::PrintStatus(void)
{
	char	s[120];
	static	lastcall;

	if (lastcall == time(0))
		return;

	lastcall = time(0);

	if (EscapeOn)
		DPuts(2, 3, "[ESC]", 0x0700);
	else
		DPuts(2, 3, "     ", 0x0700);

	sprintf(s, "[%4dk]", coreleft()/1024);
	DPuts(72, 3, s, (coreleft()<102400L)?0x3700:0x0700);
}

void TScreen::SetEsc(int l)
{
	EscapeOn = l;
}
