#include <alloc.h>
#include <conio.h>
#include <ctype.h>
#include <dir.h>
#include <fcntl.h>
#include <io.h>
#include <share.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "cdefs.h"
#include "errors.h"
#include "pktdef.h"
#include "cmfix.h"

#include "ver.h"


TGroup::TGroup(void)
{
	memset(this, 0, sizeof(TGroup));

	grpf = fopen(FUtil.CMPath("GRPLST.CM"), "rb");
	if (grpf == NULL)
		return;

	mbrf = fopen(FUtil.CMPath("GRPMBR.CM"), "rb");
	if (mbrf == NULL)
	{
		fclose(grpf);
		grpf = NULL;
		return;
	}

	DGroup		grp;
	DLink		mbr;
	TGroupRec	*gr;
	TNodeLink	*nl;

	fread(&grp, sizeof(grp), 1, grpf); 	// get rid of 1st record

	while (fread(&grp, sizeof(grp), 1, grpf) != NULL)
	{
		if (data == NULL)
			gr = data = new TGroupRec;
		else
		{
			gr->next = new TGroupRec;
			gr = gr->next;
		}

		strcpy(gr->group, grp.group);
		while (grp.members)
		{
			fseek(mbrf, grp.members, SEEK_SET);

			if (fread(&mbr, sizeof(mbr), 1, mbrf) != NULL)
			{
				if (gr->l == NULL)
					gr->l = nl = new TNodeLink;
				else
				{
					nl->next = new TNodeLink;
					nl = nl->next;
				}

				memcpy(&nl->sz, &mbr.sz, sizeof(sysznnp));
				nl->Status = mbr.Status;
			}
			else
				mbr.Next = 0;

			grp.members = mbr.Next;
		}
	}

	fclose(grpf);
	fclose(mbrf);
}

TGroup::~TGroup(void)
{
	delete data;
}

int TGroup::IsInList(char *g, znnp *z)
{
	TGroupRec	*gr = data;

	while (gr != NULL)
	{
		if (!stricmp(gr->group, g))
			return gr->l->IsInList(z);

		gr = gr->next;
	}

	return 0;
}

int TGroup::IsInList(char *g, char *z)
{
	TGroupRec	*gr = data;

	while (gr != NULL)
	{
		if (!stricmp(gr->group, g))
			return gr->l->IsInList(z);

		gr = gr->next;
	}

	return 0;
}

int TGroup::IsInAGroup(znnp *z)
{
	TGroupRec	*gr = data;

	while (gr != NULL)
	{
		if (gr->l->IsInList(z))
			return 1;
		gr = gr->next;
	}

	return 0;
}

int TGroup::IsInAGroup(char *z)
{
	TGroupRec	*gr = data;

	while (gr != NULL)
	{
		if (gr->l->IsInList(z))
			return 1;
		gr = gr->next;
	}

	return 0;
}




TCMFix::TCMFix(void)
{
	TFixAlias	*fl;
	word		num, n, lp, pp;


	memset(this, 0, sizeof(TCMFix));

	FILE *fptr = fopen(FUtil.CMPath("FIXALIAS.CM"), "rb");
	if (fptr != NULL)
	{
		char	s[40];

		while (fread(s, 35, 1, fptr) != NULL)
		{
			if (Alias == NULL)
				fl = Alias = new TFixAlias;
			else
			{
				fl->next = new TFixAlias;
				fl = fl->next;
			}

			memset(fl, 0, sizeof(TFixAlias));
			strcpy(fl->alias, s);
		}
	}
	fclose(fptr);

	CmdName = new char[40];
	CmdLine = new char[40];

	Group = new TGroup;

	memassert(Group);
}

TCMFix::~TCMFix(void)
{
	delete	Alias;
	delete	Group;

	delete	CmdName;
	delete	CmdLine;
}



#pragma argsused
int TCMFix::Process(char *file, char *curnetpath)
{
	FILE		*fptr;
	_netmail	net, out;
	znnp		afrom, ato;
	tm 			*tmn;
	time_t		tmnt;
	char		outname[50], *msgbuf;


	fptr = _fsopen(file, "rb", SH_DENYNONE);
	fread(&net, sizeof(net), 1, fptr);

	if (net.attrib & Local || net.attrib & Recd || net.attrib & Sent)
	{
		fclose(fptr);
		return 0;
	}

	if (!IsCMFixAlias(net.touser))
	{
		fclose(fptr);
		return 0;
	}

	msgbuf = new char[ filelength(fileno(fptr)) ];
	memassert(msgbuf);

	fread(msgbuf, 1, filelength(fileno(fptr)), fptr);
	fclose(fptr);

	Znnp.FindAddr(&net, msgbuf, &afrom, &ato, NULL);

	if (!Znnp.IsOurAddr(&ato))
	{
		delete msgbuf;
		return 0;
	}

	nmp++;
	memcpy(&aorig, &afrom, sizeof(znnp));
	memset(&out, 0, sizeof(out));

	strcpy(out.fromuser, "CMFix");
	strcpy(out.touser, net.fromuser);
	strcpy(out.subject, "CatMail Fix Request Reply");

	time(&tmnt);
	tmn = localtime(&tmnt);
	strftime(out.datetime, 80, "%d %b %y  %H:%M:%S", tmn);

	out.destnode = net.orignode;
	out.destnet = net.orignet;
	out.orignode = net.destnode;
	out.orignet = net.destnet;

	out.attrib|= Priv|Local|KillSent;

	GenNet.NextFree(outname, 0);
	newmsg = fopen(outname, "wb");


	fwrite(&out, sizeof(struct _netmail), 1, newmsg);

#ifdef RegisteredTo
	fprintf(newmsg, "\001PID: "_pid" %04d\r", RegistrationNumber);
#else
	fprintf(newmsg, "\001PID: "_pid" UNREGISTERED\r");
#endif

	fprintf(newmsg, "\001INTL %d:%d/%d %d:%d/%d\r",
		afrom.zone, afrom.net, afrom.node,
		ato.zone, ato.net, ato.node);

	if (ato.point)
		fprintf(newmsg, "\001FMPT %d\r", ato.point);

	if (afrom.point)
		fprintf(newmsg, "\001TOPT %d\r", afrom.point);

	memcpy(&MsgFor, &afrom, sosz);
	ProcessMsg(&net, msgbuf);

	fprintf(newmsg, "\r--- CatMail FIX v"_ver"\r");
	fclose(newmsg);

	unlink(file);
	delete msgbuf;
	return 0;
}


int TCMFix::Scan(void)
{
	cprintf("Processing inbound CatMail AreaFix messages...\r\n");

	ScanNetmail();

	cprintf("   Processed %d CMFix messages.\r\n\n", nmp);
	if (nmp)  Log->elog("FIX", '+', "%d CMFix messages processed in total", nmp);

	return OK;
}




// You can call this function with FILE *newmsg open to a blank file and
// with msgbuf set to the location of some alterations you want done with
// CMFix.  Ato is the "message from" field.

int TCMFix::ProcessMsg(_netmail *net, char *msgbuf)
{
	char	*point, *p, *pn;
	int	leave = 0, lp;
	char	cmd;
	char	f;

	char	thing[100];


	cprintf("   CMFix received from %s (%s)... ", net->fromuser,
		Znnp.ZnnpToStr(NULL, &MsgFor));

	Log->elog("FIX", '+', "CMFix message from %s at %s", net->fromuser, Znnp.ZnnpToStr(NULL, &MsgFor));


	fprintf(newmsg, "==== CatMail FIX v"_ver";  AreaFix Robot for CatMail  =========================\r\r");

#ifdef BETA
	fprintf(newmsg, "--] Please Note!!  CatMail FIX is still in beta stages of testing and  because\r");
	fprintf(newmsg, "                   of this, it may be unstable.  Please make sure your request\r");
	fprintf(newmsg, "                   has  been successfully compeleted, and report any bugs  you\r");
	fprintf(newmsg, "                   may find to Roger Nesbitt @ 3:770/220 or your SysOp.\r");
	fprintf(newmsg, "===============================================================================\r\r");
#endif


	strcpy(thing, net->subject);
	point = strchr(thing, ' ');
	if (point)
		*point = 0;

	if (cfg->cmfixsecure && !Group->IsInAGroup(&MsgFor))
	{
		fprintf(newmsg, "Your node (%s) is not set up on this system to use CMFix.\r", Znnp.ZnnpToStr(NULL, &MsgFor));
		fprintf(newmsg, "Talk to the SysOp of this system about obtaining a CMFix connection.\r\r");
		cprintf("unknown node.\r\n");
		Log->elog("FIX", '!', "Node %s is unknown to CMFix", Znnp.ZnnpToStr(NULL, &MsgFor));
		return 0;
	}

	if (!Password->Check(&MsgFor, thing))
	{
		fprintf(newmsg, "Invalid CMFix password '%s'.  Talk to the SysOp about passwords.\r\r", thing);
		cprintf("invalid password.\r\n");
		Log->elog("FIX", '!', "Node %s supplied an invalid password %s", Znnp.ZnnpToStr(NULL, &MsgFor), thing);
		return 0;
	}


	point = msgbuf;
	while (!leave && *point)
	{
		while (*point == '\r')
			point++;
		p = point;
		while (*point != '\r' && *point)
			point++;
		if (*point)
			*point++ = 0;
		else
			leave = 1;


		if (!strnicmp(p, "--- ", 4) || !strcmp(p, "---"))
			break;
		if (p[0] == 1)
			continue;
		if (strchr(p, '>'))
			continue;

		while (isspace(*p) && *p)
			p++;

		strcpy(thing, p);		// thing = current line
		p = thing;
		while (!isspace(*p) && *p)
			p++;

		*p++ = 0;		// thing = keyword, p = line
		strcpy(CmdName, thing);
		strcpy(CmdLine, p);

		if (strchr("+-%", CmdName[0]))
		{
			cmd = CmdName[0];
			strcpy(CmdName, thing+1);
		}
		else
			cmd = '+';

		if (cmd == '%')
		{
			if (!stricmp(CmdName, "help"))
				CmdHelp();

			else if (!stricmp(CmdName, "list") ||
				 !stricmp(CmdName, "query") ||
				 !stricmp(CmdName, "unlinked"))
				CmdList();

			else if (!stricmp(CmdName, "pause") ||
				 !stricmp(CmdName, "resume"))
				CmdPauseResume();

			else if (!stricmp(CmdName, "create"))
				CmdCreate();

			else if (!stricmp(CmdName, "delete"))
				CmdDelete();

			else if (!stricmp(CmdName, "+all"))
				AreaAdd("ALL");

			else if (!stricmp(CmdName, "-all"))
				AreaDelete("ALL");

			else if (!stricmp(CmdName, "from") && Group->IsInList("MANAGER", &MsgFor))
			{
				Znnp.StrToZnnp(p, &MsgFor);
				fprintf(newmsg, "\r\r *** Changed maintenance address to %s\r\r",
					Znnp.ZnnpToStr(NULL, &MsgFor));
			}

			else
				fprintf(newmsg, "\rUnknown command %s\r\r", thing);
		}
		else if (cmd == '+')
			AreaAdd(CmdName);
		else if (cmd == '-')
			AreaDelete(CmdName);
	}

	cprintf("done.\r\n");

	return OK;
}

inline void TCMFix::InitArea(void)
{
	fseek(Area->areafile, sizeof(DAreaCfg), SEEK_SET);
}

TAreaCfg *TCMFix::GetNextArea(void)
{
	TAreaCfg *ac;

	do {
		ac = Area->GetAreaCfg(0xffffffffL);
	}
	while (ac != NULL && !Group->IsInList(ac->Group, &MsgFor));

	return ac;
}

TAreaCfg *TCMFix::GetMyNextArea(void)
{
	TAreaCfg *ac;

	do {
		ac = Area->GetAreaCfg(0xffffffffL);
	}
	while (ac != NULL && !ac->Links->IsInList(&MsgFor));

	return ac;
}


int TCMFix::WildCardMatch(char *str, char *wildstr)
{
	char	*point, *p;

	point = str;
	p = wildstr;

	while (*p)
	{
		if (*p == '*')
			return 1;
		if (*p == '?')
		{
			if (*point)
				point++;
			p++;
			continue;
		}
		if (toupper(*p) != toupper(*point))
			return 0;
		if (*point)
			point++;
		p++;
	}

	return (*point == 0);
}

int TCMFix::IsCMFixAlias(char *s)
{
	TFixAlias	*a;

	if (!stricmp(s, "cmfix") || !stricmp(s, "areafix"))
		return 1;

	a = Alias;
	while (a)
	{
		if (!stricmp(s, a->alias))
			return 1;
		a = a->next;
	}
	return 0;
}



/*********** TCMFix::CmdX commands **************************************/

void TCMFix::CmdHelp(void)
{
	FILE	*fptr;
	char	thing[130];

	fptr = fopen(FUtil.CMPath("CMFIX.HLP"), "r");
	while (fgets(thing, 120, fptr) != NULL)
	{
		if (thing[strlen(thing)-1] == '\n')
			thing[strlen(thing)-1] = '\r';

		fputs(thing, newmsg);
	}
	fclose(fptr);
	fprintf(newmsg, "\r\r");
}


void TCMFix::CmdList(void)
{
	TAreaCfg	*a;

	fprintf(newmsg, "\r\rThe following is a list of message areas that %s ", Znnp.ZnnpToStr(NULL, &MsgFor));

	switch (toupper(CmdName[0]))
	{
	  case 'L':
		fputs("has access to:\r\r\r", newmsg);
		break;

	  case 'Q':
		fputs("is joined to:\r\r\r", newmsg);
		break;

	  case 'U':
		fputs("is NOT joined to:\r\r\r", newmsg);
		break;
	}


	fprintf(newmsg, "  %-40s %-11s %-11s    %-8s\r",
		"Tag Name", "SL Name", "Group", "Status");
	fprintf(newmsg, "==============================================================================\r");


	InitArea();

	while ( (a = GetNextArea()) != NULL )
	{
		switch (toupper(CmdName[0]))
		{
		  case 'L':
			break;

		  case 'Q':
			if (!a->Links->IsInList(&MsgFor))
				continue;
			break;

		  case 'U':
			if (a->Links->IsInList(&MsgFor))
				continue;
			break;
		}


		EMsgStatus ms = (EMsgStatus)a->Links->IsInList(&MsgFor);

		fprintf(newmsg, "  %-40s %-11s %-11s   [%6s]\r",
			a->isusenet?strlwr(a->Tag):a->Tag, a->SLArea, a->Group,
			ms == MSNone ? "" : ms == MSActive ? "Joined" : "Paused");
	}

	fprintf(newmsg, "\r\r");
}


void TCMFix::CmdCreate(void)
{
	char	s[255];
	char	NewTag[60], NewGroup[10], NewSLArea[20];


	if (!Group->IsInList("MANAGER", &MsgFor))
	{
		fprintf(newmsg, "\r\rYou do not have sufficent rights to create a new area.\r\r");
		return;
	}

	strupr(CmdLine);
	SUtil.aword(s, 1, CmdLine);
	s[40] = 0;
	strcpy(NewTag, s);

	SUtil.aword(s, 2, CmdLine);
	s[8] = 0;
	strcpy(NewSLArea, s);

	SUtil.aword(s, 3, CmdLine);
	s[8] = 0;
	strcpy(NewGroup, s);

	if (!*NewTag)
	{
		fprintf(newmsg, "\r\rNo area name specified to create!\r\r");
		return;
	}

	if (strchr(NewTag, '*') || strchr(NewTag, '?'))
	{
		fprintf(newmsg, "\r\rWildcards are not permitted in new area names.\r\r");
		return;
	}

	TAreaCfg *a = Area->GetTagArea(NewTag);

	if (a != NULL)
	{
		fprintf(newmsg, "\rMessage area %s already exists, therefore can't create new area.\r", NewTag);
		return;
	}

	if (Area->MakeNewArea(NewTag, NULL, 0, *NewSLArea ? NewSLArea : NULL,
										  *NewGroup ? NewGroup : NULL) != NULL)

		fprintf(newmsg, "Message area %s successfully created.\r", NewTag);
	else
		fprintf(newmsg, "Unable to create message area %s.\r", NewTag);
}

void TCMFix::CmdDelete(void)
{
	char	Tag[40];

	if (!Group->IsInList("MANAGER", &MsgFor))
	{
		fprintf(newmsg, "\r\rYou do not have sufficent rights to delete an area.\r\r");
		return;
	}

	strupr(CmdLine);
	SUtil.aword(Tag, 1, CmdLine);

	TAreaCfg *a = Area->GetTagArea(Tag);

	if (a == NULL)
	{
		fprintf(newmsg, "\rMessage area %s does not exist, so it cannot be deleted.\r", Tag);
		return;
	}

	DAreaCfg 	ac;

	fseek(Area->areafile, -sizeof(DAreaCfg), SEEK_CUR);
	fread(&ac, sizeof(DAreaCfg), 1, Area->areafile);

	ac.Deleted = 255;

	fseek(Area->areafile, -sizeof(DAreaCfg), SEEK_CUR);
	fwrite(&ac, sizeof(DAreaCfg), 1, Area->areafile);

	fprintf(newmsg, " * Message area %s successfully deleted.\r", Tag);
}



void TCMFix::CmdPauseResume(void)
{
	TAreaCfg	*AreaCfg;
	int		NumMatched = 0, NotLinked = 0;
	char		area[255];

	SUtil.aword(area, 2, CmdLine);

	int		All = !stricmp(area, "ALL");
	int		Pause = toupper(CmdLine[1] == 'P');

	InitArea();

	while ( (AreaCfg = GetNextArea()) != NULL )
	{
		if ( All ||
		     (area[0]=='#' && WildCardMatch(AreaCfg->Group, &area[1])) ||
		     WildCardMatch(AreaCfg->Tag, area) ||
		     WildCardMatch(AreaCfg->SLArea, area) )
		{
			long 	ThisLink = Area->LastAreaDCfg.Links;
			DLink   a, oa;

			if (!AreaCfg->Links->IsInList(&MsgFor))
			{
				NotLinked = 1;
				continue;
			}

			fseek(Area->linkfile, ThisLink, SEEK_SET);
			while (fread(&a, sizeof(DLink), 1, Area->linkfile) != NULL)
			{
				if (MsgFor == a.sz.z)
					break;
				memcpy(&oa, &a, sizeof(DLink));
				ThisLink = a.Next;
				fseek(Area->linkfile, ThisLink, SEEK_SET);
			}

			if (MsgFor != a.sz.z)
			{
				fprintf(newmsg, " ! Node is already unlinked from area %s.\r", area);
				break;
			}

			fseek(Area->linkfile, ThisLink, SEEK_SET);
			a.Status = Pause?MSPaused:MSActive;
			fwrite(&a, sizeof(DLink), 1, Area->linkfile);

			fprintf(newmsg, " * %s area %s.\r", Pause?"Paused":"Resumed", AreaCfg->Tag);

			NumMatched++;
		}
	}

	if (!NumMatched && NotLinked)
		fprintf(newmsg, " ! Your node is not joined to area %s.\r", area);

	if (!NumMatched && !NotLinked)
		fprintf(newmsg, " ! Unable to find area %s.\r", area);
}



void TCMFix::AreaAdd(char *area)
{
	TAreaCfg	*AreaCfg;
	int		NumMatched = 0, AlreadyLinked = 0;
	int		All = !stricmp(area, "ALL");


	InitArea();

	while ( (AreaCfg = GetNextArea()) != NULL )
	{
		if ( All ||
		     (area[0]=='#' && WildCardMatch(AreaCfg->Group, &area[1])) ||
		     WildCardMatch(AreaCfg->Tag, area) ||
		     WildCardMatch(AreaCfg->SLArea, area) )
		{
			long	ThisLink, UseLink;
			DLink	a, b;

			ThisLink = UseLink = 0L;

			if (AreaCfg->Links->IsInList(&MsgFor))
			{
				AlreadyLinked = 1;
				continue;
			}

			if (Area->LastAreaDCfg.Links > 0)
			{
				ThisLink = Area->LastAreaDCfg.Links;

				fseek(Area->linkfile, ThisLink, SEEK_SET);
				while (fread(&a, sizeof(DLink), 1, Area->linkfile) != NULL)
				{
					if (a.Next == 0)
						break;
					ThisLink = a.Next;
					fseek(Area->linkfile, ThisLink, SEEK_SET);
				}
			}

			fseek(Area->linkfile, sizeof(DLink), SEEK_SET);
			while (fread(&b, sizeof(DLink), 1, Area->linkfile) != NULL)
			{
				if (b.Status == MSNone)
				{
					UseLink = ftell(Area->linkfile)-sizeof(DLink);
					break;
				}
			}

			if (UseLink == 0)
			{
				fseek(Area->linkfile, 0L, SEEK_END);
				UseLink = ftell(Area->linkfile);
			}
			else
				fseek(Area->linkfile, UseLink, SEEK_SET);

			memset(&b, 0, sizeof(DLink));
			memcpy(&b.sz.z, &MsgFor, sosz);
			b.Status = MSActive;
			fwrite(&b, sizeof(DLink), 1, Area->linkfile);

			if (ThisLink)
			{
				fseek(Area->linkfile, ThisLink, SEEK_SET);
				a.Next = UseLink;
				fwrite(&a, sizeof(DLink), 1, Area->linkfile);
			}
			else
			{
				int	oldpos = ftell(Area->areafile);

				Area->LastAreaDCfg.Links = UseLink;
				fseek(Area->areafile, oldpos-sizeof(DAreaCfg), SEEK_SET);
				fwrite(&Area->LastAreaDCfg, sizeof(DAreaCfg), 1, Area->areafile);

				fseek(Area->areafile, oldpos, SEEK_SET);
			}

			fprintf(newmsg, " * Linked to area %s.\r", AreaCfg->Tag);

			NumMatched++;
		}
	}

	if (!NumMatched && AlreadyLinked)
		fprintf(newmsg, " ! Node is already linked to area %s.\r", area);

	if (!NumMatched && !AlreadyLinked)
		fprintf(newmsg, " ! Unable to find area %s.\r", area);
}


void TCMFix::AreaDelete(char *area)
{
	TAreaCfg	*AreaCfg;
	int		NumMatched = 0, NotLinked = 0;
	int		All = !stricmp(area, "ALL");


	InitArea();

	while ( (AreaCfg = GetNextArea()) != NULL )
	{
		if ( All ||
		     (area[0]=='#' && WildCardMatch(AreaCfg->Group, &area[1])) ||
		     WildCardMatch(AreaCfg->Tag, area) ||
		     WildCardMatch(AreaCfg->SLArea, area) )
		{
			long 	ThisLink = Area->LastAreaDCfg.Links;
			long 	OldLink = 0L;
			DLink   a, oa;

			if (!AreaCfg->Links->IsInList(&MsgFor))
			{
				NotLinked = 1;
				continue;
			}

			fseek(Area->linkfile, ThisLink, SEEK_SET);
			while (fread(&a, sizeof(DLink), 1, Area->linkfile) != NULL)
			{
				if (MsgFor == a.sz.z)
					break;
				memcpy(&oa, &a, sizeof(DLink));
				OldLink = ThisLink;
				ThisLink = a.Next;
				fseek(Area->linkfile, ThisLink, SEEK_SET);
			}

			if (MsgFor != a.sz.z)
			{
				fprintf(newmsg, " ! Node is already unlinked from area %s.\r", area);
				break;
			}

			if (OldLink == 0L)
			{
				if (a.Next == 0)
				{
					fseek(Area->linkfile, ThisLink, SEEK_SET);
					memset(&a, 0, sizeof(a));
					fwrite(&a, sizeof(DLink), 1, Area->linkfile);

					long oldpos = ftell(Area->areafile);

					Area->LastAreaDCfg.Links = 0L;
					fseek(Area->areafile, oldpos-sizeof(DAreaCfg), SEEK_SET);
					fwrite(&Area->LastAreaDCfg, sizeof(DAreaCfg), 1, Area->areafile);

					fseek(Area->areafile, oldpos, SEEK_SET);
				}
				else
				{
					fseek(Area->linkfile, a.Next, SEEK_SET);
					fread(&a, sizeof(DLink), 1, Area->linkfile);
					fseek(Area->linkfile, ThisLink, SEEK_SET);
					fwrite(&a, sizeof(DLink), 1, Area->linkfile);
				}
			}
			else
			{
				fseek(Area->linkfile, ThisLink, SEEK_SET);
				a.Status = MSNone;
				fwrite(&a, sizeof(DLink), 1, Area->linkfile);
				fseek(Area->linkfile, OldLink, SEEK_SET);
				oa.Next = a.Next;
				fwrite(&oa, sizeof(DLink), 1, Area->linkfile);
			}

			fprintf(newmsg, " * Unlinked from area %s.\r", AreaCfg->Tag);

			NumMatched++;
		}
	}

	if (!NumMatched && NotLinked)
		fprintf(newmsg, " ! Node is already unlinked from area %s.\r", area);

	if (!NumMatched && !NotLinked)
		fprintf(newmsg, " ! Unable to find area %s.\r", area);
}



int TCMFix::ScanNetmail(void)
{
	ffblk		f;
	char		path[75], file[75];
	int		n;
	DNetPath	np;
	FILE		*fptr;

	fptr = fopen(FUtil.CMPath("NETPATH.CM"), "rb");

	sprintf(path, "%s*.MSG", cfg->netpath);

	do
	{
		n = findfirst(path, &f, 0);
		while (!n)
		{
			sprintf(file, "%s%s", cfg->netpath, f.ff_name);
			path[strlen(path)-5] = 0;

			if (Process(file, path))
				break;
			n = findnext(&f);
		}

		*path = 0;

		if (fptr != NULL && fread(&np, sizeof(np), 1, fptr))
			sprintf(path, "%s*.MSG", np.NetPath);
	}
	while (*path);

	if (fptr != NULL)
		fclose(fptr);

	return OK;
}
