#include <alloc.h>
#include <conio.h>
#include <ctype.h>
#include <dir.h>
#include <io.h>
#include <process.h>
#include <string.h>

#include "cdefs.h"
#include "icfg.h"

#include "cmarc.h"
#include "cminet.h"
#include "cmcmi.h"
#include "cmdisp.h"



TInternet::TInternet(void)
{
	memset(this, 0, sizeof(TInternet));

	icfg = new _icfg;

	FILE *fptr = fopen(FUtil.CMPath("ICFG.CM"), "rb");
	if (fptr == NULL)
		return;

	fread(icfg, sizeof(_icfg), 1, fptr);
	fclose(fptr);

	Log->log(LDebug, ' ', "TInternet: Internet system initialised");

	if (icfg->enabled)
		ObjectIsSetup = 1;
}


TInternet::~TInternet(void)
{
	delete icfg;
	ObjectIsSetup = 0;
}


int TInternet::LookUpAlias(char *MsgTo, char *PersonTo)
{
	if (!ObjectIsSetup)
		return 1;

	FILE *fptr = fopen(FUtil.CMPath("ALIAS.CM"), "rb");
	if (fptr != NULL)
	{
		char		ns[200];
		TNameAlias	na;

		sprintf(ns, "%s@%s", MsgTo, icfg->address);

		while (fread(&na, sizeof(na), 1, fptr) != NULL)
		{
			if (!stricmp(na.inetname, MsgTo) || !stricmp(ns, MsgTo))
			{
				fclose(fptr);
				strcpy(PersonTo, na.user);

				Log->log(LDebug, ' ', "LookUpAlias: %s -> %s", MsgTo, PersonTo);

				// 7Apr1998
				// ignore aliases that have ignore set,
				// or unsupported ones like forward, redirect and bounce.

				if (na.atype == A_Ignore ||
					na.atype == A_Forward ||
					na.atype == A_Redirect ||
					na.atype == A_Bounce)
					return 2;

				if (na.atype == A_MailList)
					return 3;

				return 0;
			}
		}

		fclose(fptr);
	}


	char	*p;

	strcpy(PersonTo, MsgTo);

	if ( (p = strchr(PersonTo, '@')) != NULL )
		*p = 0;

	while ( (p = strchr(PersonTo, '.')) != NULL )
		*p = ' ';

	strupr(PersonTo);

	return 0;
}

int TInternet::Uncompress(char *oldpath, char *newpath)
{
	FILE 	*fptr, *gptr;
	char	s[20];

	fptr = fopen(oldpath, "rb");

	fread(s, 1, 12, fptr);
	s[12] = 0;

	// 7Apr1998
	// make all "?unbatch" into "cunbatch" so we can parse it.
	s[3] = 'c';

	if (strcmp(s, "#! cunbatch\n"))
	{
		fclose(fptr);
		strcpy(newpath, oldpath);
		return 0;
	}

	char *buf = new char[32000];
	memassert(buf);

	tmpnam(newpath);
	newpath[strlen(newpath)-1] = 'Z';

	gptr = fopen(newpath, "wb");

	int n;

	do
	{
		n = fread(buf, 1, 32000, fptr);
		fwrite(buf, 1, n, gptr);
	}
	while (n == 32000);

	fclose(fptr);
	fclose(gptr);

	delete buf;

	TScreen *ArcScr = new TScreen;
	if (ArcScr != NULL)
		ArcScr->Save();

	TArchive 	*Archive = new TArchive;
	TDArchive 	*a;
	char		cmdline[120];

	a = Archive->CompressType(newpath);
	Archive->Interpret(a->CmdExtract, newpath, "", cmdline);

	system(cmdline);

	delete Archive;

	if (ArcScr != NULL)
	{
		ArcScr->Load();
		delete ArcScr;
	}

	newpath[strlen(newpath)-1] = 0;

	if (access(newpath, 0))
	{
		Log->error("Uncompression failed on packet %s", newpath);
		return 1;
	}

	return 0;
}

int SeparateName(char *ss, char *MsgFrom, char *PersonFrom)
{
	char *p, *pp, *s;
	int Start, lp;

	s = new char[strlen(ss)+1];
	memassert(s);
	strcpy(s, ss);

	while (strchr("\r\n", s[strlen(s)-1]) != NULL)
		s[strlen(s)-1] = 0;

	p = strchr(s, '@');
	if (p == NULL)
	{
		delete s;
		return 1;
	}

	while (*p != ' ' && *p != '<' && *p != '(' && p >= s)
		p--;

	p++;

	strcpy(MsgFrom, p);

	while ( (pp = strchr(MsgFrom, ' ')) != NULL )
		*pp = 0;

	while (*MsgFrom && strchr(")>", MsgFrom[strlen(MsgFrom)-1]) != NULL)
		MsgFrom[strlen(MsgFrom)-1] = 0;

	Start = p - s;

	p = s;
	pp = PersonFrom;

	for (lp=0; lp<Start; lp++)
		*pp++ = *p++;

	p += strlen(MsgFrom);

	while (*p)
		*pp++ = *p++;

	*pp = 0;

	p = PersonFrom;
	pp = s;

	while (*p == ' ' || *p == '(' || *p == '<')
		p++;

	while (*p)
		*pp++ = *p++;

	*pp = 0;

	while (*s && strchr(" )>(<", s[strlen(s)-1]) != NULL)
		s[strlen(s)-1] = 0;

	strcpy(PersonFrom, s);

	if (!*s)
	{
		strcpy(PersonFrom, MsgFrom);
		p = strchr(PersonFrom, '@');
		if (p != NULL)
			*p = 0;

		if ( (p = strchr(PersonFrom, '.')) != NULL )
			*p = ' ';
	}

	delete s;
	return 0;
}



int TInternet::ProcessEMail(char *path, char *MsgTo, char *SystemFrom)
{
	char 	PersonTo[80], MsgFrom[80], PersonFrom[80],
			s[255], Subject[80], *p, *pp, *buf;

	long	size;
	const	bufsize = 32000;
	int		n;

	FILE	*fptr, *iptr;


	fptr = fopen(path, "rb");
	if (fptr == NULL)
	{
		Log->error("[I005] Unable to open corresponding data file %s", path);
		return 1;
	}

	Log->log(LVerbose, ' ', "Processing EMail D file %s", path);

	buf = new char[bufsize];
	memassert(buf);

	*buf = *MsgFrom = *PersonFrom = 0;

	while (fgets(s, 255, fptr) != NULL)
	{
		if (!strnicmp(s, "From: ", 6))
		{
			SeparateName(&s[6], MsgFrom, PersonFrom);
		}

		if (!strnicmp(s, "Subject: ", 9))
		{
			strncpy(Subject, &s[9], 80);
			SUtil.KillRet(Subject);
		}

		if (*s == '\n')
			break;

		if (s[strlen(s)-1] == '\n')
			s[strlen(s)-1] = '\r';

		if (strlen(s)+strlen(buf) < bufsize)
			strcat(buf, s);
	}

	if (*s != '\n')
	{
		fclose(fptr);
		Log->error("[I006] Blank message %s?", path);

		delete buf;
		return 1;
	}

	if (!*MsgFrom)
		strcpy(MsgFrom, "UNKNOWN SENDER");

	strcat(buf, "\r");

	sysznnp		sfrom;
	sfrom.t = DInternet;
	strcpy(sfrom.s, SystemFrom);

	if (*cfg->inetlogfile)
		iptr = fopen(cfg->inetlogfile, "a");

	pp = p = strcpy(s, MsgTo);

	long 	fptrpos = ftell(fptr);
	TCmi	*Cmi;


	do
	{
		while (!isspace(*pp) && *pp)
			pp++;

		if (*pp != 0)
		{
			*pp = 0;
			pp++;
			while (isspace(*pp))
				pp++;
		}

		n = LookUpAlias(p, PersonTo);
		if (n == 2)
		{
			delete buf;

			if (*cfg->inetlogfile)
				fclose(iptr);

			fclose(fptr);

			return 1;
		}

		if (n == 3)
		{
			TAreaCfg *a = Area->GetSLArea(PersonTo);

			if (a != NULL)
				Cmi = new TCmi(&sfrom, a->Tag, a->SLArea);
			else
				Cmi = new TCmi(&sfrom, PersonTo, PersonTo);
		}

		if (n == 0)
			Cmi = new TCmi(&sfrom, "MAIL", "MAIL");

		Cmi->InitMessage();
		Cmi->FillMessageHeader(Util.GetFTSCDate(NULL), PersonFrom, MsgFrom, PersonTo, p, Subject, -1, 1, NULL, strlen(buf));

		Cmi->SetCRType(CR0a);
		Cmi->MakeMessage(buf);
		fseek(fptr, fptrpos, SEEK_SET);
		Cmi->MakeMessage(fptr);
		size = Cmi->SaveMessage();

		Log->log(LDebug, ' ', " -> Email sent to %s (%s) from %s (%s)", PersonTo, p, PersonFrom, MsgFrom);

		if (*cfg->inetlogfile)
			fprintf(iptr, "IN,%s,%s,%s,%s,%ld\n",
				PersonTo, p, PersonFrom, MsgFrom, size);

		p = pp;

		Cmi->CloseCMI();
		delete Cmi;

		nimp++;
	}
	while (*p != 0);

	delete buf;

	fclose(fptr);
	if (*cfg->inetlogfile)
		fclose(iptr);

	return 0;
}


TAreaCfg *TInternet::GetInternetArea(char **s, char *from_i, int CreateNew)
{
	char			*p, *first = NULL;
	TAreaCfg        *a;


	if (!**s || *s == NULL)
		return NULL;

	do
	{
		p = strchr(*s, ',');
		if (p != NULL)
			*p = 0;

		if (CreateNew && first == NULL)
		{
			first = new char[strlen(*s)+1];
			strcpy(first, *s);
		}

		a = Area->GetTagArea(*s);

		if (p != NULL)
			*s = p + 1;
		else
			*s+= strlen(*s);
	}
	while (a == NULL && **s);

	if (a == NULL && CreateNew)
	{
		sysznnp *out = new sysznnp(from_i);
		a = Area->MakeNewArea(first, out, 1);
		delete out;
	}


	if (first != NULL) delete first;

	return a;
}

int TInternet::ProcessUsenet(char *path, char *SystemFrom)
{
	FILE	*uf;
	char	MsgFrom[80], PersonFrom[80], Subject[80], Date[40];
	char	*nglist;
	long	len;
	long	HdrStart, MsgStart;
	int		kill;
	_nodestats	ns;


	char	*s, *buf;
	const	size = 32000;

	buf = new char[size];

	if (buf == NULL)
	{
		Log->error("No memory to import usenet messages");
		return 1;
	}

	Log->log(LVerbose, ' ', "Processing Usenet D file %s", path);

	uf = fopen(path, "rb");

	TCmi		Cmi;
	TAreaCfg	*AreaCfg;
	long		msgs = 0;

	nglist = NULL;

	sysznnp	sfrom;
	sfrom.t = DInternet;
	strcpy(sfrom.s, SystemFrom);

	Stats->ReadStats(&sfrom, &ns);

	long 	flen = filelength(fileno(uf)) / 1024;

	ns.k_from+= flen;
	Stats->MainStats.KImport+= flen;

	while (fgets(buf, 120, uf) != NULL)
	{
		sscanf(buf, "%*s %*s %ld", &len);

		s = buf;
		kill = 0;
		nglist = NULL;

		while (fgets(s, size, uf) != NULL)
		{
			if (*s && s[strlen(s)-1] == '\n')
				s[strlen(s)-1] = '\r';

			if (s[0] == '\r' || s[0] == '\n')
				break;

			if (!strncmp(s, "From: ", 6))
			{
				SeparateName(&s[6], MsgFrom, PersonFrom);
			}
			else if (!strncmp(s, "Subject: ", 9))
			{
				strncpy(Subject, &s[9], 80);
				SUtil.KillRet(Subject);
			}
			else if (!strncmp(s, "Newsgroups: ", 12))
			{
				nglist = new char[strlen(s)];
				strcpy(nglist, &s[12]);
				SUtil.KillRet(nglist);
			}
			else if (!strncmp(s, "Control: ", 9) && icfg->killcontrols)
			{
				Log->log(LAverage, '?', "Control message found in %s; deleted.", path);
				kill = 1;
			}

			else if (!strncmp(s, "Date: ", 6))
			{
				strncpy(Date, &s[6], 40);
				SUtil.KillRet(Date);
			}

			s+= strlen(s);
		}

		if (nglist == NULL || kill)
		{
			if (nglist != NULL)
				delete nglist;

			fseek(uf, len-strlen(buf), SEEK_CUR);
			continue;
		}

		Stats->MainStats.MImport++;
		ns.msg_from++;
		msgs++;

		char *p = nglist;

		AreaCfg = GetInternetArea(&p, SystemFrom, 1);

		if (AreaCfg == NULL)
		{
			Log->log(LVerbose, '?', "ProcessUsenet: No conference available for message %ld in %s", msgs, path);

			fseek(uf, len-strlen(buf), SEEK_CUR);
			delete nglist;
			continue;
		}

		MsgStart = ftell(uf);

		while (AreaCfg)
		{
			Cmi.OpenCMI(&sfrom, AreaCfg->Tag, AreaCfg->SLArea);
			Cmi.InitMessage();
			Cmi.FillMessageHeader(Util.GetFTSCDate(NULL), PersonFrom, MsgFrom, "All", "", Subject, len, 1, SystemFrom, strlen(buf));

			Cmi.SetCRType(CR0a);
			Cmi.MakeMessage(buf);
			Cmi.MakeMessage(uf);

			Cmi.SaveMessage();
			Cmi.CloseCMI();

			AreaCfg = GetInternetArea(&p, SystemFrom, 0);
			if (AreaCfg != NULL)
				fseek(uf, MsgStart, SEEK_SET);

			nump++;
		}

		delete nglist;
	}

	Log->log(LDebug, ' ', " -> Packet contains %ld messages, %ldkbytes", msgs, flen);

	fclose(uf);
	Stats->WriteStats(&ns);
	delete buf;

	return 0;
}

int TInternet::ProcessXFile(char *path)
{
	typedef enum { TNone, TEMail, TUsenet } msgtype_t;

	msgtype_t	msgtype = TNone;

	FILE *fptr = fopen(path, "r");
	if (fptr == NULL)
	{
		Log->error("[I002] Unable to open %s for reading", path);
		return 1;
	}

	char *MsgTo, s[120], newpath[80], SystemFrom[80], *p;

	MsgTo = new char[1024];

	*MsgTo = 0;

	while (fgets(s, 120, fptr) != NULL)
	{
		if (s[0] == 'U')
		{
			p = &s[2];
			while (!isspace(*p) && *p)
				p++;
			while (isspace(*p))
				p++;
			strncpy(SystemFrom, p, 80);
			SUtil.KillRet(SystemFrom);
		}

		if (s[0] == 'C')
		{
			if (s[strlen(s)-1] == '\n')
				s[strlen(s)-1] = 0;

			if (!strnicmp(&s[2], "rmail", 5))
			{
				msgtype = TEMail;
				strncpy(MsgTo, &s[8], 1024);
				if (strrchr(MsgTo, '!') != NULL)
					strcpy(MsgTo, strrchr(MsgTo, '!') + 1);
			}
			else if (!strnicmp(&s[2], "rnews", 5))
				msgtype = TUsenet;
			else
			{
				fclose(fptr);
				Log->error("[I003] Unknown mail command [%s] in %s", &s[2], path);
				return 1;
			}
		}

	}

	fclose(fptr);

	if (msgtype == TNone)
	{
		Log->error("[I004] Message %s has no receipient", path);
		return 1;
	}

	sprintf(s, "%s!", icfg->uucpname);
	if (!strnicmp(MsgTo, s, strlen(s)))
	{
		// newpath is a temp variable

		sprintf(newpath, "%s@%s", &MsgTo[strlen(s)], icfg->address);
		strcpy(MsgTo, newpath);
	}


	path[strlen(path)-1] = 'D';

	if (Uncompress(path, newpath))
		return 1;

	int	n;

	if (msgtype == TEMail)
		n = ProcessEMail(newpath, MsgTo, SystemFrom);
	else
		n = ProcessUsenet(newpath, SystemFrom);

	if (n == 0)
	{
		unlink(newpath);
		unlink(path);
		path[strlen(path)-1] = 'X';
		unlink(path);
	}

	delete MsgTo;

	return n;
}


void TInternet::Scan(void)
{
	if (!ObjectIsSetup)
	{
		cprintf("   Internet commands disabled.\r\n\n");
		return;
	}

	nimp = nump = 0;

	cprintf("Scanning for inbound internet messages... ");

	char	Path[70];
	ffblk	f;

	int	x = wherex();

	FILE 	*fptr = fopen(FUtil.CMPath("HOSTS.CM"), "rb");
	THosts 	h;

	if (fptr == NULL)
	{
		cprintf("\r\n");
		Log->error("There are no internet hosts defined.  Inbound skipped.");
		cprintf("\r\n");
		return;
	}

	while (fread(&h, sizeof(h), 1, fptr) != NULL)
	{
		strupr(h.name);

		gotoxy(x, wherey());
		cprintf("%-12s", h.name);

		sprintf(Path, "%s%s\\*.X", icfg->spoolpath, h.name);

		int n = findfirst(Path, &f, 0);
		while (!n)
		{
			sprintf(Path, "%s%s\\%s", icfg->spoolpath, h.name, f.ff_name);
			Log->log(LDebug, ' ', "Processing X file %s", Path);
			ProcessXFile(Path);
			n = findnext(&f);
		}
	}

	fclose(fptr);

	gotoxy(x, wherey());
	cprintf("            \r\n   Imported %d internet messages.\r\n", nimp);
	cprintf("   Imported %d usenet messages.\r\n\n", nump);
}
