// next error S020

#include <alloc.h>
#include <conio.h>
#include <dos.h>
#include <dir.h>
#include <errno.h>
#include <io.h>
#include <process.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "errors.h"
#include "defs.h"
#include "cmdline.h"
#include "cfgdef.h"
#include "swap\spawno.h"
#include "cdefs.h"
#include "ver.h"

#include "cmarc.h"
#include "cmcpkt.h"
#include "cmext.h"
#include "cmfix.h"
#include "cmdisp.h"
#include "cmnet.h"
#include "cmpkt.h"
#include "cminet.h"
#include "cmcfg.h"


int		errorlevel = 0;


void 	CheckEsc(void);
CmdType ProcessCmdLine(void);
int 	ProcessCmdSwitches(void);
_cmb 	*FindCmbStruct(char *);
void 	MakeDirs(void);


unsigned	_stklen = 16384;


inline void CheckEsc(void)
{
	if (kbhit())
	{
		char ch = getch();
		if (ch == 27)
		{
			cprintf("       [ESC] pressed, startup aborted\r\n");
			Sys.bye(6);
		}
	}
}


#pragma argsused
int main(int argc, char *argv[])
{
	TScreen	Screen;


	cprintf("\r\n  CatMail v"_ver" Initialisation\r\n");

	cprintf("       Press [ESC] to abort CatMail Startup\r\n");

#ifdef OVERLAY
	long ovrmem = 256L*1024L;
	cprintf("       Overlayed version\r\n");
	if (!_OvrInitEms(0, 0, ovrmem/16))
		cprintf("       Using EMS for overlay buffer\r\n");
	else
	{
		if (!_OvrInitExt(0, 0))
			cprintf("       Using XMS for overlay buffer\r\n");
		else
			cprintf("       Using conventional memory for overlay buffer\r\n");
	}
#endif

	randomize();

	if (_osmajor < 3)
	{
		cprintf("\r\nError [S002]:  DOS v3.0 or greater is required to run CatMail.\r\n");
		exit(18);
	}


	ReadConfiguration();

	if (Sys.IsAnotherCopyRunning()
	#ifdef BETA
			&& stricmp(argv[1], "/s-")
	#endif
			)
	{
		cprintf("\r\nError [S001]:  Only one copy of CatMail may be run at a time.\r\n");
		cprintf("               If another copy is NOT running or is crashed, delete CMINUSE.FLG.\r\n");
		exit(19);	// this is allowed!
	}

	if (!access(FUtil.CMPath("CMNOT.NOW"), 0))
	{
		cprintf("\r\nCMNOT.NOW exists in CatMail directory.  CatMail not executed.\r\n");
		exit(24);
	}

	Sys.CopyIsRunning();


	cprintf("       Processing command line switches\r\n");
	ProcessCmdSwitches();

	cprintf("       Reading configuration files\r\n");
	Log 		= new TLog;
	Area 		= new TArea;
	Stats		= new TStats;
	Password 	= new TPassword;
	Searchlight = new TSearchlight;

	memassert(Password);

	MakeDirs();

	init_SPAWNO("c:\\", cfg->swapto);

	Screen.SetEsc(1);

	Log->log(' ', "");
	Log->log(LTerse, '+', "CatMail v"_ver" Startup");

	if (!FUtil.IsEnoughHDSpace(cfg->workpath, 524288))
	{
		Log->error("[S009] At least 512k of free disk space required for CM to run");
		Sys.bye(21);
	}

	if (Route.ReadCfg() != OK)
		Sys.bye(40);

	CheckEsc();

	if (Area->AreaIDXNeedsUpdate)
	{
		cprintf("       Sorting area list\r\n");
		Area->Sort();
		cprintf("       Generating area index file\r\n");
		Area->GenIDX();
	}

	CheckEsc();

	Screen.Setup();
	Screen.PrintStatus();

	TExternal	External;
	TArchive	*Archive;
	TPkt		*Pkt;
	TToss		*Toss;
	TNet		*Net;
	TCMFix		*Fix;
	TInternet	*Inet;


	word 	CommandNumber = 0;
	CmdType n;

	do
	{
		Screen.PrintStatus();
		n = ProcessCmdLine();
		switch (n)
		{
		  case CmdNone:
		  case CmdOk:
			break;

		  case CmdFidoIn:
			if (!cfg->fidoenabled)
			{
				cprintf("   FidoNet commands disabled.\r\n");
				break;
			}

			Net = new TNet; Net->Import(); delete Net;
			Archive = new TArchive; Archive->Extract(); delete Archive;
			Pkt = new TPkt; Pkt->Import(); delete Pkt;

			External.Call("Import", 0);

			Sys.CallArchiveOut = 1;
			break;

		  case CmdInetIn:
			Inet = new TInternet; Inet->Scan(); delete Inet;

			External.Call("Import", 0);

			Sys.CallArchiveOut = 1;
			break;

		  case CmdOut:
			External.Call("Export ExportMail", 0);

			Sys.CallArchiveOut = 1;
			break;

		  case CmdAll:
			External.Call("Export ExportMail", 0);

			if (cfg->fidoenabled)
			{
				Archive = new TArchive; Archive->Extract(); delete Archive;
				Pkt = new TPkt; Pkt->Import(); delete Pkt;
				Net = new TNet; Net->Import(); delete Net;
			}

			Inet = new TInternet; Inet->Scan(); delete Inet;
			External.Call("Import", 0);

			Sys.CallArchiveOut = 1;
			break;

		  case CmdPack:
			if (!cfg->fidoenabled)
			{
				cprintf("   FidoNet commands disabled.\r\n");
				break;
			}

			Net = new TNet; Net->Export(); delete Net;
			Sys.CallArchiveOut = 1;
			break;

		  case CmdMail:
			External.Call("ExportMail", 0);
			Net = new TNet; Net->Import(); delete Net;
			Sys.CallMailIn = 1;
			break;

		  case CmdCMFix:
			if (!cfg->fidoenabled)
			{
				cprintf("   FidoNet commands disabled.\r\n");
				break;
			}

			Fix = new TCMFix; Fix->Scan(); delete Fix;
			break;

		  case CmdHelp:
			Screen.Help();
			break;

		  case CmdInfo:
			Screen.CMInfo();
			break;

		  case CmdPok:
			cprintf("\r\n *!* This program has Pok v1.00 installed. *!*\r\n");
			break;

		  default:
			Log->baderror("Unknown command (%d) returned to CMMAIN.C switch loop!", (int)n);
		}

		CommandNumber++;
	}
	while (n != CmdNone);

	if (Sys.CallMailIn)
		External.Call("ImportMail", 0);

	if (cfg->fidoenabled && Sys.CallArchiveOut)
	{
		Toss = new TToss;
		Toss->TossCMI();
		delete Toss;

		Archive = new TArchive;
		Archive->Archive();
		delete Archive;
	}

	Sys.bye(errorlevel);
	return 0;
}


CmdType ProcessCmdLine(void)
{
	int		lp, pp;
	static	int	upto = 0;
	char		thing[45], cmdline[255];

	CmdType	       	cmd = CmdNone;


	if (_argc == 1)
		return CmdHelp;

	upto++;
	while (upto < _argc && strchr("/-_", *_argv[upto]))
		upto++;

	if (upto >= _argc)
		return CmdNone;

	Log->log(LVerbose, ' ', "Executing command line instruction [%s]", _argv[upto]);

	lp = 0;
	while (*def[lp].cmd)
	{
		if (!stricmp(_argv[upto], def[lp].cmd))
		{
			cmd = def[lp].ret;
			cmdline[0] = 0;
			for (pp=upto; pp<upto+def[lp].nopts; pp++)
			{
				strcat(cmdline, _argv[pp]);
				strcat(cmdline, " ");
			}
			upto+= def[lp].nopts;
			break;
		}
		lp++;
	}

	if (cmd == CmdNone)
	{
		struct _cmb *cmb = FindCmbStruct(_argv[upto]);

		if (cmb == NULL)
		{
			Log->error("[S020] Unknown command '%s'.", _argv[upto]);
			cprintf("            Type CM.EXE for command line parameter help.\r\n");
			Sys.bye(20);
		}

		lp = 0;
		while (*cmb->cmd[lp++].cmd)
		{
			if (!stricmp(cmb->cmd[lp].cmd, _argv[upto]))
			{
				*cmdline = 0;
				for (pp=upto; pp<upto+(cmb->cmd[lp].nopts)+1; pp++)
				{
					strcat(cmdline, _argv[pp]);
					strcat(cmdline, " ");
				}
				upto+= cmb->cmd[lp].nopts;
				break;
			}
		}

		strcpy(thing, cmb->progid);
		strcat(thing, ".CME");

		TExternal External;
		External.CallCMB(thing, cmdline);

		return CmdOk;
	}

	return cmd;
}


int ProcessCmdSwitches(void)
{
	int	lp;

	for (lp=1; lp<_argc; lp++)
	{
		if (*_argv[lp] == '_')
			continue;
		if (*_argv[lp] == '-' || *_argv[lp] == '/')
		{
			switch (_argv[lp][1])
			{
/*			  case 'c':
			    if (_argv[lp][2] == 0)
			    {
//				strcpy(cfgfile, _argv[lp+1]);
				_argv[lp+1][0] = '_';
			    }
			    else
				strcpy(cfgfile, &_argv[lp][2]);

			    if (access(cfgfile, 0))
			    {
				Log->error("Configuration file %s does not exist!", cfgfile);
				return FAILURE;
			    }
			    break;

			  case 'l':
			    if (_argv[lp][2] == 0)
			    {
//				strcpy(logname, _argv[lp+1]);
				_argv[lp+1][0] = '_';
			    }
			    else
				strcpy(logname, &_argv[lp][2]);
			    break;

*/			  default:
			    Log->error("Unrecognised switch -%c", _argv[lp][1]);
			    return FAILURE;
			}
		}
	}
	return OK;
}


_cmb *FindCmbStruct(char *cmd)
{
	int	n, lp, hand;
	struct	ffblk	f;
	char	l[120];

	static	struct	_cmb	cmb;


	sprintf(l, "%s*.cmb", cfg->cmpath);
	n = findfirst(l, &f, 0);
	while (!n)
	{
		sprintf(l, "%s%s", cfg->cmpath, f.ff_name);

		FILE *fptr = fopen(l, "rb");
		if (fptr == NULL)
		{
			n = findnext(&f);
			continue;
		}

		fread(&cmb, sizeof(cmb), 1, fptr);
		fclose(fptr);

		if (stricmp(cmb.id, "CMB1"))
		{
			n = findnext(&f);
			continue;
		}

		lp = 0;
		while (cmb.cmd[lp].cmd[0])
		{
			if (!stricmp(cmb.cmd[lp].cmd, cmd))
				return &cmb;

			lp++;
		}

		n = findnext(&f);
	}
	return NULL;
}

void MakeDirs(void)
{
	int	n = 0;
	char	*thing = new char[75];

	strcpy(thing, cfg->workpath);
	strcat(thing, "OUTPKT\\");

	n |= FUtil.MakePath(cfg->filespath);
	n |= FUtil.MakePath(cfg->arcnetpath);
	n |= FUtil.MakePath(cfg->packpath);
	n |= FUtil.MakePath(cfg->workpath);
	n |= FUtil.MakePath(thing);
	n |= FUtil.MakePath(cfg->binkpath);
	n |= FUtil.MakePath(cfg->netpath);
	n |= FUtil.MakePath(cfg->dupepath);

	delete thing;

	if (n)
	{
		Log->error("Unable to create one or more paths.  Check CMSetup paths.");
		Sys.bye(4);
	}
}
