#include <alloc.h>
#include <conio.h>
#include <dir.h>
#include <dos.h>
#include <io.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "pktdef.h"
#include "errors.h"
#include "cdefs.h"
#include "crc32.h"
#include "id.h"

#include "cmdb.h"
#include "cmnet.h"
#include "cmpkt.h"


#define g db; getch()
#define min(x, y) ((x) < (y) ? (x) : (y))



TPkt::TPkt(void)
{
	memset(this, 0, sizeof(TPkt));

	msgbuf = new char[cfg->msgbufsize+8];
	LastReadLine = new char[90];

	pkthdr = new _pkthdr;
	msghdr = new _msghdr;
	cmmsg = new _cmmsg;
	area = new char[40];
	pktname = new char[75];

	orig = new znnp;
	dest = new znnp;

	SentDupeToSys = NetInRun = 0;

	LastSLArea = new char[42];
	LastBadArea = new char[42];

	memassert(LastBadArea);
	memassert(msgbuf);

	strcpy(LastSLArea, " ");
	strcpy(LastBadArea, " ");
}


TPkt::~TPkt(void)
{
	delete msgbuf;
	delete LastReadLine;

	delete pkthdr;
	delete msghdr;
	delete cmmsg;
	delete area;
	delete pktname;

	delete orig;
	delete dest;

	delete LastSLArea;
	delete LastBadArea;
}


void TPkt::CopyPKTs(void)
{
	struct	ffblk	ff;
	int	n;
	char	thing[120], thingy[120];

	sprintf(thing, "%s*.pkt", cfg->netpath);
	n = findfirst(thing, &ff, 0);

	while (!n)
	{
		sprintf(thing, "%s%s", cfg->netpath, ff.ff_name);
		sprintf(thingy, "%s%s", cfg->workpath, ff.ff_name);
		if (FUtil.CopyFile(thing, thingy) == OK)
			unlink(thing);
		else
			unlink(thingy);
		n = findnext(&ff);
	}

}


word		nmp = 0;


int TPkt::Import(void)
{
	int		n;
	ffblk 		ff;

	TDataBase	*DBase;

	DBase = new TDataBase;
	DBase->InboundLook();
	delete DBase;

	cprintf("Processing inbound FidoNet mail...\r\n");

	CopyPKTs();

	sprintf(pktname, "%s*.PKT", cfg->workpath);

	n = findfirst(pktname, &ff, 0);
	while (!n)
	{
		strcpy(LastBadArea, " ");

		sprintf(pktname, "%s%s", cfg->workpath, ff.ff_name);
		switch (n = ReadPKT())
		{
			case OK:
				break;

			case INVALID_PACKET:
			case PKT_NOT_SUPPORTED:
				break;

			default:
				return n;
		}

		n = findnext(&ff);
	}

	if (CMIfile != NULL)
	{
		fclose(CMIfile);
		CMIfile = NULL;
	}

	cprintf("   Processed %d FidoNet messages.\r\n\n", nmp);
	Log->log(LVerbose, '+', "Processed %d FidoNet messages", nmp);

	if (NetInRun)
	{
		TNet	Net;

		cprintf("\r\n\n");
		Net.Import();
	}

	return OK;
}


void TPkt::CheckCatPkt(void)
{
	int		n;
	TDataBase	DataBase;


	n = DataBase.LookUp(orig);
	if (n == -1)
	{
		DataBase.Send(orig, AreYouCatMail);
		DataBase.Add(orig, Possible);
	}
	else if (DataBase.CatRec->known == BadSystem)
	{
		Log->error("Unable to receive packets from this system.  Contact author.");
		unlink(pktname);
		sound(3300);
		sleep(2);
		nosound();
		return;
	}
}

inline word swap(word a) { return (a << 8) | (a >> 8); }

int TPkt::ReadPKT(void)
{
	char	*point;
	int		lp;

	memset(pkthdr, 0, sizeof(_pkthdr));

	NumMsgsImported = 0;

	pktfile = fopen(pktname, "rb");
	if (pktfile == NULL)
	{
		Log->error("Unable to open packet %s for reading.", pktname);
		return FAILURE;
	}

	setvbuf(pktfile, NULL, _IOFBF, cfg->msgcachesize);

	fread(pkthdr, sizeof(_pkthdr), 1, pktfile);

	if (!(pkthdr->cw & 1) || pkthdr->cw != swap(pkthdr->cwval))
		ConvType1Pkthdr();

	if (pkthdr->destzone == 0 || pkthdr->destzone == 65535)
		pkthdr->destzone = pkthdr->destzone_old;

	if (pkthdr->origzone == 0 || pkthdr->origzone == 65535)
		pkthdr->origzone = pkthdr->origzone_old;

	if (!pkthdr->destzone)
	{
		for (lp=0; lp<15; lp++)
		{
			if (cfg->address[lp].net == pkthdr->destnet &&
				cfg->address[lp].node == pkthdr->destnode)
			{
				pkthdr->destzone = cfg->address[lp].zone;
				break;
			}
		}
		if (!pkthdr->destzone) pkthdr->destzone = cfg->address[0].zone;
	}

	if (!pkthdr->origzone)
		pkthdr->origzone = pkthdr->destzone;

	orig->zone = pkthdr->origzone;
	orig->net = pkthdr->orignet;
	orig->node = pkthdr->orignode;
	orig->point = pkthdr->origpoint;

	dest->zone = pkthdr->destzone;
	dest->net = pkthdr->destnet;
	dest->node = pkthdr->destnode;
	dest->point = pkthdr->destpoint;

	cprintf("   %s from %s : processing area ",
		FUtil.FileOnly(pktname), Znnp.ZnnpToStr(NULL, orig));

	Log->log(LDebug, ' ', "TPkt: Processing packet %s from %s",
		FUtil.FileOnly(pktname), Znnp.ZnnpToStr(NULL, orig));

	if (pkthdr->ident != 2)
	{
		fclose(pktfile);
		FUtil.RenameToXXX(pktname, "BAD");

		cprintf("\r\n   ! Packet is either grunged or non-fido compliant.\r\n");
		Log->log('!', "Packet %s is either grunged or non-fido compliant", FUtil.FileOnly(pktname));

		return INVALID_PACKET;
	}

	if (pkthdr->prodspec[0] == 174)	// a catmail packet!
		CheckCatPkt();

	if (!Znnp.IsOurAddr(dest))  	// if pkt not destined for here.
	{
		CheckRoutedMsg();
		return OK;
	}

	if (cfg->secure && Password->Check(orig, pkthdr->password) == FAILURE)
	{
		fclose(pktfile);

		FUtil.RenameToXXX(pktname, "BAD");

		cprintf("\r\n   ! Invalid Password on Packet Header.\r\n");
		Log->log('!', "Packet %s has an invalid header password", FUtil.FileOnly(pktname));
		return OK;
	}

	sysznnp *sz = new sysznnp(*orig);
	Stats->ReadStats(sz, &ns);
	delete sz;

	long kfrom = filelength(fileno(pktfile))/1024;

	if (kfrom == 0)
		kfrom = 1;

	ns.k_from+= kfrom;
	Stats->MainStats.KImport+= kfrom;

	int	ErrorOccured = 0;

	while (!feof(pktfile) && !ErrorOccured)
	{
		if (GetType2MsgHeader())
			break;

		nmp++;
		NumMsgsImported++;
		ns.msg_from++;
		Stats->MainStats.MImport++;

		if (!stricmp(area, "MAIL"))
			ProcType2Netmail();
		else
			ErrorOccured = ProcType2EchoMail();
	}

	fclose(pktfile);

	Stats->WriteStats(&ns);

	unlink(pktname);

	TidyTMP("PKTTEMP.$$0", "PKT", pktname);
	TidyTMP("PKTTEMP.$$1", "SEC", pktname);
	TidyTMP("PKTTEMP.$$2", "DUP", pktname);

	Log->log(LDebug, ' ', " -> Contained %d messages, %ldkbytes", NumMsgsImported, kfrom);

	cputs("\r\n");
	return OK;
}


void TPkt::ConvType1Pkthdr(void)
{
	int	lp, n;

	pkthdr->majrevision = 0;
	memset(&pkthdr->password, 0, 8);
	memset(&pkthdr->prodspec, 0, 4);

	pkthdr->destzone = pkthdr->origzone = 0;

	for (lp=n=0; lp<15; lp++)
	{
		if (pkthdr->destnode == cfg->address[lp].node &&
		    pkthdr->destnet  == cfg->address[lp].net)
		{
			pkthdr->origzone = pkthdr->destzone = cfg->address[lp].zone;
			break;
		}

		if (pkthdr->destnet == cfg->address[lp].net)
			n = lp;
	}

	if (pkthdr->origzone == 0 || pkthdr->destzone == 0)
		pkthdr->origzone = pkthdr->destzone = cfg->address[n].zone;

	pkthdr->auxnet = 0;
	pkthdr->cwval = pkthdr->cw = 0;
	pkthdr->productcode_h = 0;
	pkthdr->minrevision = 0;
	pkthdr->origpoint = pkthdr->destpoint = 0;
}


int TPkt::GetType2MsgHeader(void)
{
	memset(msghdr, 0, sizeof(_msghdr));
	memset(cmmsg,  0, sizeof(_cmmsg));

	if (!fread(msghdr, sizeof(_msghdr), 1, pktfile))
		return 1;

	FUtil.fngets(cmmsg->datetime, 20, pktfile);

	if (strlen(cmmsg->datetime) == 16)
		strcat(cmmsg->datetime, ":00");

	if (strlen(cmmsg->datetime) != 19)
		strcpy(cmmsg->datetime, "01 Jan 98  01:00:00");

	if (*cmmsg->datetime < '0' || *cmmsg->datetime > '3')
		strcpy(cmmsg->datetime, "01 Jan 98  01:00:00");

	FUtil.fngets(cmmsg->touser,   38, pktfile);
	FUtil.fngets(cmmsg->fromuser, 38, pktfile);
	FUtil.fngets(cmmsg->subject,  74, pktfile);

	char	s[10];

	fread(s, 1, 5, pktfile);
	s[5] = 0;
	if (strcmp(s, "AREA:"))		// if it's netmail
	{
		strcpy(area, "MAIL\r");
		Log->log(LDebug, '*', "TPkt: NetMail found in packet at offset %ld", ftell(pktfile));
		fseek(pktfile, -5L, SEEK_CUR);
	}
	else
		FUtil.frgets(area, 120, pktfile);

	if (!*area)
		strcpy(area, "MAIL\r");
	area[strlen(area)-1] = 0;

	cmmsg->size = -1;
	memcpy(&cmmsg->orig, orig, sosz);
	strcpy(cmmsg->id, CMI_MSG_ID);

	return 0;
}



void TPkt::ProcType2Netmail(void)
{
	word		BytesRead;
	char   		*point;
	_netmail	net;
	znnp		tonode, fromnode;
	char		path[75];
	FILE		*fptr;

	BytesRead = FUtil.fngets(msgbuf, cfg->msgbufsize, pktfile);

	cprintf("%-14s", "Netmail");
	gotoxy(wherex()-14, wherey());

	point = msgbuf;
	while (*point == '\r')
		point++;

	while (*point == 1)
	{
		while (*point != 0 && *point != '\r')
			point++;
		while (*point == '\r')
			point++;
	}

	if (!*point && cfg->killblank)
	{
		Log->log('+', "Blank inbound netmail message deleted");
		cprintf("\r\n     Blank inbound netmail message deleted\r\n");
		return;
	}

	memset(&net, 0, sizeof(net));

	if (!stricmp(cmmsg->touser, "SYSOP"))
		strcpy(cmmsg->touser, cfg->sysop);

	strcpy(net.touser, cmmsg->touser);
	strcpy(net.fromuser, cmmsg->fromuser);
	strcpy(net.subject, cmmsg->subject);
	strcpy(net.datetime, cmmsg->datetime);
	net.destnode = msghdr->destnode;
	net.orignode = msghdr->orignode;
	net.destnet = msghdr->destnet;
	net.orignet = msghdr->orignet;
	net.cost = msghdr->cost;
	net.attrib = msghdr->attrib;

	net.attrib&= (
		Priv | Recd | Sent | FileAttached |
		InTransit | Orphan | KillSent |
		Hold );		// strips local too remember

	if (cmmsg->fileattach)
		net.attrib|= FileAttached;

	GenNet.NextFree(path, 0);

	if (!stricmp(net.fromuser, "sysop") && cfg->rennet)
		strcpy(net.fromuser, cfg->sysop);

	Znnp.FindAddr(&net, msgbuf, &fromnode, &tonode, NULL);

	fptr = fopen(path, "wb");
	fwrite(&net, sizeof(_netmail), 1, fptr);
	fwrite(msgbuf, 1, BytesRead==65535?cfg->msgbufsize:BytesRead, fptr);

	while (BytesRead == 65535)
	{
		BytesRead = FUtil.fngets(msgbuf, cfg->msgbufsize, pktfile);
		fwrite(msgbuf, 1, BytesRead==65535?cfg->msgbufsize:BytesRead, fptr);
	}

	fclose(fptr);

	if (*cfg->rescanfile)
		close(_creat(cfg->rescanfile, 0));

	NetInRun = 1;
}


int TPkt::ProcType2EchoMail(void)
{
	TAreaCfg	*AreaCfg;
	_dupepkt	dupepkt;

	char	s[42];
	word	BytesRead;

	memset(&dupepkt, 0, sizeof(dupepkt));
	memset(&FUtil.LastBracket, 0, 30);

	BytesRead = FUtil.fngets(msgbuf, cfg->msgbufsize, pktfile);

	if (BytesRead == 0)
		Log->error("Odd?  Bytesread = 0 at %ld", ftell(pktfile));

	if (stricmp(area, AreaCfg->Tag))
	{
		AreaCfg = Area->GetTagArea(area);

		if (AreaCfg == NULL)
		{
			sysznnp *out = new sysznnp(*orig);

			AreaCfg = Area->MakeNewArea(area, out);

			delete out;

			if (AreaCfg == NULL)
			{
				if (stricmp(area, LastBadArea))
				{
					cprintf("\r\n   ! Received unknown message area %s - skipping.", area);
					Log->log('!', "%s received with unknown tag area %s", FUtil.FileOnly(pktname), area);
					strcpy(LastBadArea, area);
				}

				TossBadMessage("$$0", BytesRead);

				return 0;
			}
		}
	}

	if (AreaCfg->freeze && !*AreaCfg->SLArea)
	{
		TossBadMessage("$$0", BytesRead);
		return 0;
	}


	znnp anywhere;
	Znnp.FindAddr(NULL, msgbuf, &cmmsg->origin, &anywhere, &dupepkt);

	if (cmmsg->origin.net == 0)
		Znnp.StrToZnnp(FUtil.LastBracket, &cmmsg->origin);


	if (AreaCfg->dupe == Yes || (AreaCfg->dupe == Default &&
					  cfg->dupecheck))
	{
		if (dupepkt.msgidcrc == 0)
			dupepkt.msgidcrc = Util.Crc32(msgbuf, min(BytesRead, 1024));

		char	*crcs = new char[200];

		strcpy(crcs, cmmsg->datetime);
		strcat(crcs, cmmsg->touser);
		strcat(crcs, cmmsg->fromuser);
		strcat(crcs, cmmsg->subject);
		strcat(crcs, area);
		dupepkt.crc = Util.Crc32(crcs);

		delete crcs;
	}


	strcpy(s, area);
	s[14] = 0;
	cprintf("%-14s", s);
	gotoxy(wherex()-14, wherey());

	if (AreaCfg->secure == Yes || (AreaCfg->secure == Default &&
				       cfg->secure))
	{
		if (!AreaCfg->Links->IsInList(orig))
		{
			TossBadMessage("$$1", BytesRead);
			Log->error("Secure area %s violation from %s", area, Znnp.ZnnpToStr(NULL, orig));
			printf("\r\n     Node is not in the area's downlink list.\r\n");
			return 0;
		}
	}

	if (AreaCfg->dupe == Yes ||
	    (AreaCfg->dupe == Default && cfg->dupecheck) )
	{
		if (!*AreaCfg->SLArea)
			sprintf(s, "!%07lX", Util.Crc32(AreaCfg->Tag) & 0x0fffffff);
		else
			strcpy(s, AreaCfg->SLArea);

		switch (Dupe.DupeCheck(s, &dupepkt))
		{
		  case OK:
			break;
		  case NO_MEM:
			return FAILURE;
		  case FAILURE:
			ns.dupes++;
			cputs("\r\n");
			Log->error("Dupe check failure on packet %s from %s",
				pktname, Znnp.ZnnpToStr(NULL, orig));

			TossBadMessage("$$2", BytesRead);

			if (!SentDupeToSys)
			{
				SentDupeToSys = 1;
				MailToSysOp(DUPE);
			}

			return 0;
		}
	}

	if (stricmp(LastSLArea, AreaCfg->SLArea))
	{
		if (CMIfile != NULL)
			fclose(CMIfile);

		if (*AreaCfg->SLArea)
			strcpy(LastSLArea, AreaCfg->SLArea);
		else
			sprintf(LastSLArea, "!%07lX", Util.Crc32(AreaCfg->Tag) & 0x0fffffff);

		sprintf(s, "%s%s.CMI", cfg->workpath, LastSLArea);

		if (access(s, 0))
		{
			CMIfile = fopen(s, "w+b");

			_cmhdr	cmhdr;

			memset(&cmhdr, 0, sizeof(cmhdr));
			strcpy(cmhdr.id, CMI_HDR_ID);
			strcpy(cmhdr.tag, AreaCfg->Tag);
			strcpy(cmhdr.slarea, AreaCfg->SLArea);

			cmhdr.Origin.t = DFidoNet;
			memcpy(&cmhdr.Origin.z, orig, sosz);

			if (AreaCfg->import == No || !*AreaCfg->SLArea)
				cmhdr.AllImported = 1;

			fwrite(&cmhdr, sizeof(cmhdr), 1, CMIfile);
		}
		else
		{
			CMIfile = fopen(s, "r+b");

			fseek(CMIfile, 0L, SEEK_END);
		}
	}

	WriteToCMI(msgbuf, CMIfile, BytesRead);

	return OK;
}


void TPkt::WriteToCMI(char *msgbuf, FILE *msgfile, word BytesRead)
{
	long	MsgStart, tmsglen;

	if (BytesRead != 65535)
		cmmsg->size = BytesRead;
	else
		MsgStart = ftell(msgfile);

	fwrite(cmmsg, sizeof(_cmmsg), 1, msgfile);
	if (msgbuf != NULL)
		fwrite(msgbuf, 1, BytesRead==65535?cfg->msgbufsize:BytesRead, msgfile);

	if (BytesRead != 65535)
		return;

	tmsglen = cfg->msgbufsize;

	while (BytesRead == 65535)
	{
		BytesRead = FUtil.fngets(msgbuf, cfg->msgbufsize, pktfile);

		if (BytesRead == 65535)
			tmsglen+= cfg->msgbufsize;
		else
			tmsglen+= BytesRead;

		fwrite(msgbuf, 1, BytesRead==65535?cfg->msgbufsize:BytesRead, msgfile);
	}

	fseek(msgfile, MsgStart, SEEK_SET);
	cmmsg->size = tmsglen;
	fwrite(cmmsg, sizeof(_cmmsg), 1, msgfile);
	fseek(msgfile, 0L, SEEK_END);
}


int TPkt::CheckRoutedMsg(void)
{
	fclose(pktfile);

	if (cfg->allowpktforward && (Route.AllowForward(dest, ForwardTo) ||
				     Route.AllowForward(orig, ForwardFor) ))
	{
		long pktout;
		sscanf(FUtil.FileOnly(pktname), "%lx", &pktout);

		CPkt.Add(dest, pktout, EchoMail);

		Log->log('@', "Routed packet %s to %s",
			FUtil.FileOnly(pktname), Znnp.ZnnpToStr(NULL, dest));

		return OK;
	}

	Log->log('@', "Incorrect addr or not allowed to forward packet %s", FUtil.FileOnly(pktname));
	FUtil.RenameToXXX(pktname, "BAD");

	cprintf("\r\n   !  Incorrect address on packet header or not allowed to forward packet.\r\n");

	return FAILURE;
}



int TPkt::TossBadMessage(char *ext, word BytesRead)
{
	FILE	*fptr;
	char	temppkt[75];


	sprintf(temppkt, "%sPKTTEMP.%s", cfg->workpath, ext);

	if (access(temppkt, 0))
	{
		close(_creat(temppkt, 0));
		fptr = fopen(temppkt, "wb");
		fwrite(pkthdr, sizeof(struct _pkthdr), 1, fptr);
	}
	else
		fptr = fopen(temppkt, "ab");

	fwrite(msghdr, sizeof(_msghdr), 1, fptr);

	fwrite(cmmsg->datetime, strlen(cmmsg->datetime)+1,	1, fptr);
	fwrite(cmmsg->touser, 	strlen(cmmsg->touser)+1,	1, fptr);
	fwrite(cmmsg->fromuser, strlen(cmmsg->fromuser)+1, 	1, fptr);
	fwrite(cmmsg->subject, 	strlen(cmmsg->subject)+1, 	1, fptr);

	fprintf(fptr, "AREA:%s\r", area);

	fwrite(msgbuf, 1, BytesRead==65535?cfg->msgbufsize:BytesRead, fptr);
	while (BytesRead == 65535)
	{
		BytesRead = FUtil.fngets(msgbuf, cfg->msgbufsize, pktfile);
		fwrite(msgbuf, 1, BytesRead==65535?cfg->msgbufsize:BytesRead, fptr);
	}

	fclose(fptr);

	return OK;
}


TDupe::TDupe(void)
{
	memset(this, 0, sizeof(TDupe));
}

TDupe::~TDupe(void)
{
	if (dptropen)
		fclose(dptr);
}

int TDupe::DupeCheck(char *area, _dupepkt *dupepkt)
{
	char		thing[120];
	int		n, maxchng, lp;


	if (area == NULL)
	{
		if (dptropen)
			fclose(dptr);
		dptropen = 0;
		return OK;
	}

	if (stricmp(area, last))
	{
		sprintf(thing, "%s%s.DUP", cfg->dupepath, area);
		if (dptropen)
			fclose(dptr);
		n = maxchng = 0;
		if (access(thing, 0))
		{
			n = 1;
			dptr = fopen(thing, "w+b");
			dh.next = 0;
			dh.max = cfg->dupecheck;
			strcpy(dh.id, "CMD1");
		}
		else
		{
			dptr = fopen(thing, "r+b");
			fread(&dh, sizeof(dh), 1, dptr);
			if (dh.max > cfg->dupecheck)
			{
				if (dh.next >= cfg->dupecheck)
					dh.next = 0;
			}
			if (dh.max != cfg->dupecheck)
				maxchng = 1;
		}

		if (memalloced)
		{
			farfree(dp);
			memalloced = 0;
		}

		dp = (_dupepkt *)farcalloc(dh.max>cfg->dupecheck?dh.max:cfg->dupecheck, sizeof(_dupepkt));
		if (dp == NULL)
		{
			Log->error("Not enough memory to perform dupe checking");
			return NO_MEM;
		}
		memalloced = 1;
		strcpy(last, area);
		dptropen = 1;
		if (!n)
		{
			for (lp=0; lp<dh.max; lp++)
				fread(&dp[lp], sizeof(_dupepkt), 1, dptr);
		}
		if (maxchng)
		{
			fclose(dptr);
			dptr = fopen(thing, "w+b");
			n = 1;
			maxchng = 0;
			dh.max = cfg->dupecheck;
		}
		if (n)
		{
			fwrite(&dh, sizeof(dh), 1, dptr);
			for (lp=0; lp<dh.max; lp++)
				fwrite(&dp[lp], sizeof(_dupepkt), 1, dptr);
		}
	}

	for (lp=0; lp<dh.max; lp++)
		if (!memcmp(&dp[lp], dupepkt, sizeof(_dupepkt)))
			return FAILURE;

	n = dh.next;
	memcpy(&dp[n], dupepkt, sizeof(_dupepkt));
	dh.next++;
	if (dh.next >= dh.max)
		dh.next = 0;

	fseek(dptr, 0L, SEEK_SET);
	fwrite(&dh, sizeof(dh), 1, dptr);

	fseek(dptr, sizeof(dh)+sizeof(_dupepkt)*n, SEEK_SET);
	fwrite(&dp[n], sizeof(_dupepkt), 1, dptr);

	return OK;
}



void TPkt::MailToSysOp(int type)
{
	char	thing[120];
	struct	_cmhdr	cmhdr;
	struct	_cmmsg	cmmsg;
	char	*buf;
	FILE	*fptr;


	if (type != SHAREWARE && !cfg->rep[type])
		return;

	sprintf(thing, "%sMAIL.CMI", cfg->workpath);
	if (access(thing, 0))
	{
		fptr = fopen(thing, "wb");
		memset(&cmhdr, 0, sizeof(cmhdr));
		strcpy(cmhdr.id, CMI_HDR_ID);
		strcpy(cmhdr.tag, "MAIL");
		fwrite(&cmhdr, sizeof(cmhdr), 1, fptr);
	}
	else
	{
		fptr = fopen(thing, "r+b");
		fseek(fptr, 0, SEEK_END);
	}


	memset(&cmmsg, 0, sizeof(cmmsg));

	Util.GetFTSCDate(cmmsg.datetime);

	if (cfg->rennet)
		strcpy(cmmsg.touser, "SYSOP");
	else
		strcpy(cmmsg.touser, cfg->sysop);

	strcpy(cmmsg.fromuser, "CATMAIL");
	strcpy(cmmsg.subject, "Automated Message from CatMail");


	buf = new char[8192];
	memassert(buf);

	sprintf(buf, "This is an automated mail message to the SysOp from the CatMail Mail Processor.\r\r");

	switch (type)
	{
	  case DUPE:
		strcat(buf, "It has been reported that one or more duplicate messages have been found entering your system.  It is advised that you check your CM.LOG file and your work directory ");
		strcat(buf, "to fix any problem that may have occured, and to inform the system sending the dupes that there is something wrong with their setup.\r\r");
		break;
	  case CRITERR:
		strcat(buf, "An undefined critical error has occurred while trying to import or export some mail into or out from a Searchlight message base.  It is advised that you check your ");
		strcat(buf, "CM.LOG file, or run CatMail again and supervise its operation to check everything is functional again.  One or more Searchlight subboards may require repairing ");
		strcat(buf, "using the repair functions inside Searchlight's CONFIG, or you may have run out of disk space.\r\r");
		break;
	  case DISK_SPACE:
		strcat(buf, "CatMail has reported you have run out of disk space on either your work drive and/or on your Searchlight message base drive.  Please fix this problem and run CatMail again.\r\r");
		break;
	  case NEW_AREA:

		FILE	*tptr;

		strcat(buf, "New message area(s) have been created on your system, called:\\lc\r\r");

		tptr = fopen(FUtil.CMPath("NEWSUB.CMD"), "rb");
		while (fgets(thing, 120, tptr) != NULL)
		{
			while (thing[strlen(thing)-1] == '\r' ||
			       thing[strlen(thing)-1] == '\n')
				thing[strlen(thing)-1] = 0;

			strcat(buf, "   ");
			strcat(buf, thing);
			strcat(buf, "\r");
		}
		fclose(tptr);
		unlink(FUtil.CMPath("NEWSUB.CMD"));

		strcat(buf, "\r\\NOIt is suggested you enter CMSetup and set up these new areas to suit your requirements.\r\r");
		break;

	}

	cmmsg.size = strlen(buf);
	fwrite(&cmmsg, sizeof(cmmsg), 1, fptr);
	fwrite(buf, 1, cmmsg.size, fptr);
	fclose(fptr);

	delete buf;

	Sys.CallMailIn = 1;
}


void TPkt::TidyTMP(char *name, char *ext, char *file)
{
	char	s[80], ss[80];

	sprintf(s, "%s%s", cfg->workpath, name);

	strcpy( ss, file );
	strcpy( (ss + strlen(ss) - 3), ext );

	rename(s, ss);
}
