// up to [r005]

#include <alloc.h>
#include <ctype.h>
#include <io.h>
#include <stdio.h>
#include <string.h>

#include "cdefs.h"
#include "errors.h"

int TRoute::ReadCfg(void)
{
	FILE    *fptr;
	char    *line, key[254], CfgStr[254];
	int	CfgLine;
	int	n, lp;
	znnp	tz, otz;

	TDRoute		*dr, *fr;
	TZnnpLink	*zl;


	fptr = fopen(FUtil.CMPath("ROUTE.CFG"), "r");
	if (fptr == NULL)
	{
		Log->error("[R001] Unable to open ROUTE.CFG.  Please refer to the documentation.");
		return FAILURE;
	}

	while (fgets(CfgStr, 252, fptr) != NULL)
	{
		line = CfgStr;

		while (isspace(*line))
			line++;

		CfgLine++;

		SUtil.KillRet(line);

		if (!isalnum(*line))
			continue;

		SUtil.aword(line, 1, key);

		while (isspace(*line++));
		while (!isspace(*line++));
		while (isspace(*line++));
		line--;


		if (!stricmp(key, "route") || !stricmp(key, "fileroute"))
		{
			n = 1;
			while (1)
			{
				SUtil.aword(line, n, key);
				if (!stricmp(key, "to"))
				{
					if (SUtil.aword(line, n+2, key) != NULL)
						return FAILURE;
					if (SUtil.aword(line, n+1, key) == NULL)
						return FAILURE;
					break;
				}
				if (key == NULL)
					return FAILURE;
				n++;
			}

			SUtil.aword(line, n+1, key);

			Znnp.StrToZnnp(key, &tz, &cfg->address[0]);

			if (!Znnp.IsValidAddr(&tz))
				return FAILURE;

			if (tz.zone == 65535 || tz.net == 65535 || tz.node == 65535 || tz.point == 65535)
			{
				Log->error("[R003] May not route to node with wildcards");
				return FAILURE;
			}

			if (RouteData == NULL)
				dr = RouteData = new TDRoute;
			else
			{
				dr->next = new TDRoute;
				dr = dr->next;
			}

			dr->type = (tolower(key[0]) == 'r') ? RouteTo : FileRoute;
			zl = dr->data = new TZnnpLink;

			memcpy(&zl->z, &tz, sosz);

			memcpy(&otz, &cfg->address[0], sosz);
			for (lp=1; lp<n; lp++)
			{
				SUtil.aword(line, lp, key);
				Znnp.StrToZnnp(key, &tz, &otz);
				memcpy(&otz, &tz, sosz);
				if (!Znnp.IsValidAddr(&tz))
					return FAILURE;

				zl->next = new TZnnpLink;
				zl = zl->next;
				memcpy(&zl->z, &tz, sosz);
			}
		}

		else if (!stricmp(key, "noroute") || !stricmp(key, "hubsend") ||
			 !stricmp(key, "forwardfor") || !stricmp(key, "forwardto"))
		{
			if (RouteData == NULL)
				RouteData = dr = new TDRoute;
			else
			{
				dr->next = new TDRoute;
				dr = dr->next;
			}

			if (!stricmp(key, "noroute"))
				dr->type = NoRoute;
			if (!stricmp(key, "hubsend"))
				dr->type = HubSend;
			if (!stricmp(key, "forwardfor"))
				dr->type = ForwardFor;
			if (!stricmp(key, "forwardto"))
				dr->type = ForwardTo;

			memcpy(&otz, &cfg->address[0], sosz);

			n = 1;
			while (SUtil.aword(line, n, key) != NULL)
			{
				Znnp.StrToZnnp(key, &tz, &otz);
				memcpy(&otz, &tz, sosz);
				if (!Znnp.IsValidAddr(&tz))
					return FAILURE;

				if (dr->data == NULL)
					zl = dr->data = new TZnnpLink;
				else
				{
					zl->next = new TZnnpLink;
					zl = zl->next;
				}

				memcpy(&zl->z, &tz, sosz);
				n++;
			}
		}


		else if (!stricmp(key, "hold") || !stricmp(key, "normal") ||
			 !stricmp(key, "imm") || !stricmp(key, "crash"))
		{
			if (FlavData == NULL)
				FlavData = fr = new TDRoute;
			else
			{
				fr->next = new TDRoute;
				fr = fr->next;
			}


			switch (tolower(*key))
			{
			  case 'n': fr->type = FNormal;    break;
			  case 'h': fr->type = FHold;      break;
			  case 'c': fr->type = FCrash;     break;
			  case 'i': fr->type = FImmediate; break;
			}

			if (fr->type == FImmediate && cfg->mailertype == M_Binkley)
			{
				Log->error("[R004] Can't use Immediate routing under BinkleyTerm");
				return FAILURE;
			}

			memcpy(&otz, &cfg->address[0], sosz);

			n = 1;
			while (SUtil.aword(line, n, key) != NULL)
			{
				Znnp.StrToZnnp(key, &tz, &otz);
				memcpy(&otz, &tz, sosz);
				if (!Znnp.IsValidAddr(&tz))
					return FAILURE;

				if (fr->data == NULL)
					zl = fr->data = new TZnnpLink;
				else
				{
					zl->next = new TZnnpLink;
					zl = zl->next;
				}

				memcpy(&zl->z, &tz, sosz);

				n++;
			}
		}

		else
		{
			Log->error("[R002] Unknown command %s in ROUTE.CFG on line %d", key, CfgLine);
			return CFG_ERROR;
		}
	}
	fclose(fptr);

	return OK;
}

void TRoute::RouteCPkt(void)
{
	TDCPkt	*d = CPkt.data;

	while (d != NULL)
	{
		if (!d->routed)
		{
			memcpy(&d->unrouted, &d->to, sosz);

			DCPkt RouteType;

			if (d->DataB != NULL)
				RouteType = FileAttach;
			else
			{
				RouteType = NetMail;
				TCPktA *a = d->DataA;
				while (a != NULL)
				{
					if (a->ptype == EchoMail)
					{
						RouteType = EchoMail;
						a = NULL;
					}
					else
						a = a->next;
				}
			}

			RouteZnnp(&d->to, RouteType);

			if (d->unrouted != d->to)
				d->routed = 1;
		}

		switch (GetFlavour(&d->to))
		{
		  case FNormal:
			break;

		  case FHold:
			if (d->flavour != FImmediate)
				d->flavour = FHold;
			break;

		  case FCrash:
			d->flavour = FCrash;
			break;

		  case FImmediate:
			d->flavour = FImmediate;
			break;
		}

		d = d->next;
	}
}


RouteType TRoute::RouteZnnp(znnp *z, DCPkt PktType)
{
	TDRoute         *r = RouteData;
	RouteType	c = NoRoute;
	znnp	o, raddr;


	memcpy(&o, z, sosz);

	if (z->point)
	{
		z->point = 0;
		if (Znnp.IsOurAddr(z))
			z->point = o.point;
	}

	while (r != NULL)
	{
		if ( (r->type == RouteTo && PktType != NetMail) ||
		     (r->type == FileRoute && PktType == NetMail) ||
		     r->type == ForwardTo || r->type == ForwardFor )
		{
			r = r->next;
			continue;
		}

		TZnnpLink *zl = r->data;

		if (r->type == RouteTo || r->type == FileRoute)
		{
			memcpy(&raddr, &zl->z, sosz);
			zl = zl->next;
		}

		while (zl != NULL)
		{
			if (o.equiv(zl->z))
			{
				c = r->type;
				memcpy(z, &o, sosz);
				switch (c)
				{
				  case NoRoute:
					break;

				  case RouteTo:
				  case FileRoute:
					memcpy(z, &raddr, sosz);
					break;

				  case HubSend:
					z->node = z->point = 0;
					break;
				}

				break;
			}
			zl = zl->next;
		}

		r = r->next;
	}

	return c;
}



RouteType TRoute::GetFlavour(znnp *z)
{
	TDRoute	*r = FlavData;

	while (r != NULL)
	{
		TZnnpLink	*zl = r->data;

		while (zl != NULL)
		{
			if (z->equiv(zl->z))
				return r->type;
			zl = zl->next;
		}

		r = r->next;
	}

	return FNormal;
}

int TRoute::AllowForward(znnp *z, RouteType ftype)
{
	TDRoute *r = RouteData;

	while (r != NULL)
	{
		if (r->type != ftype)
		{
			r = r->next;
			continue;
		}

		TZnnpLink	*zl = r->data;

		while (zl != NULL)
		{
			if (z->equiv(zl->z))
				return 1;
			zl = zl->next;
		}

		r = r->next;
	}

	return 0;
}
