#include <conio.h>
#include <stdio.h>
#include <string.h>
#include <io.h>
#include <stdlib.h>
#include <share.h>

#include "cdefs.h"
#include "defs.h"
#include "cmsl.h"


struct TreeRootType
{
	long    a, listroot, c, treeroot, d;
	long    entries;
	byte	e;
};

struct TreeLeafType
{
	byte	status;
	long	left, right;
	long	last, next;
};

struct SetupHeader
{
	TreeRootType	root;
	byte			pad[116];
} sh;

struct SetupData
{
	TreeLeafType    leaf;
	byte			name_l;
	char			name[8];
	byte			path_l;
	char			path[40];
	byte			descrip_l;
	char			descrip[40];
	int				access;
	byte			attrib[3];

	union
	{
		struct
		{
			byte	subsysop_l;
			char	subsysop[25];
			bool	echomail;
			byte	postattrib[3];
			bool	visible;
			byte	pad[16];
		} a;

		struct
		{
			byte	filepath_l;
			char	filepath[38];
			bool	readonly, writeonly;
			word	free;
			int		value;
			bool	skipscan;
		} b;
	};
};





TSearchlight::TSearchlight(void)
{
	FILE		*fptr;


	if (!access("config.sl2", 0))
		strcpy(configsl2, "CONFIG.SL2");
	else
	{
		char *p, path[60];

		p = getenv("SLBBS");

		if (p == NULL)
		{
			cprintf("  ! Unable to find Searchlight configuration file.\r\n");
			cprintf("     Check your SLBBS environment variable is correctly set.\r\n");
#ifdef test
			exit(10);
#else
			Sys.bye(10);
#endif
		}

		strcpy(path, p);
		if (path[strlen(path)-1] != '\\')
			strcat(path, "\\");

		strcat(path, "CONFIG.SL2");

		if (access(path, 0))
		{
			cprintf("  ! Unable to find Searchlight configuration file.\r\n");
			cprintf("     Check your SLBBS environment variable is correctly set.\r\n");
#ifdef test
			exit(10);
#else
			Sys.bye(10);
#endif
		}

		strcpy(configsl2, path);
	}

	fptr = _fsopen(configsl2, "rb", SH_DENYNONE);
	if (fptr == NULL)
	{
		cprintf("  ! Unable to find Searchlight configuration file.\r\n");
		cprintf("     Check your SLBBS environment variable is correctly set.\r\n");
#ifdef test
			exit(10);
#else
			Sys.bye(10);
#endif
	}

	fread(&cf, sizeof(cf), 1, fptr);
	fclose(fptr);

	cf.systemname[cf.systemname_l] = 0;
	cf.sysopname[cf.sysopname_l] = 0;
	cf.progpath[cf.progpath_l] = 0;
	cf.altprogpath[cf.altprogpath_l] = 0;
	cf.datapath[cf.datapath_l] = 0;
}




int TSearchlight::FindSLArea(char *slarea)
{
	char	s[60];
	long	n;
	SetupData	sd;

	if (*cf.datapath)
		strcpy(s, cf.datapath);
	else
		strcpy(s, cf.progpath);

	strcat(s, "SUBBOARD.SL2");

	fptr = _fsopen(s, "rb", SH_DENYNONE);
	if (fptr == NULL)
	{
		cprintf("  ! Unable to find Searchlight SUBBOARD.SL2 file.\r\n");
		return -1;
	}

	strcpy(findname, slarea);

	fread(&sh, sizeof(sh), 1, fptr);


	n = sh.root.treeroot;
	while (n != 0)
	{
		fseek(fptr, sizeof(SetupHeader)+sizeof(SetupData)*(n-1), SEEK_SET);
		fread(&sd, sizeof(sd), 1, fptr);

		if (sd.leaf.status == 0)
		{
			sd.name[sd.name_l] = 0;
			if (!stricmp(slarea, sd.name))
			{
				fclose(fptr);
				return 1;
			}
		}

		if (stricmp(slarea, sd.name) < 0)
			n = sd.leaf.left;
		else
			n = sd.leaf.right;
	}


	fclose(fptr);
	return 0;
}


void TSearchlight::MakeSLName(char *tag, char *slarea, int maxlen)
{
	char	*s[8], *p;
	int		lp, n, largest, c[8], tc;

	p = tag;
	while (*p)
	{
		if (strchr("%^*+=\\|[]:;\",/?", *p) || *p >= 127)
			*p = '0';

		p++;
	}

	if (strchr(tag, '.') != NULL)
	{
		n = 1;
		tc = 0;
		s[0] = p = tag;

		while (n <= 8 && *p)
		{
			c[n-1] = 0;

			while (*p != '.' && *p)
			{
				p++;
				c[n-1]++;
				tc++;
			}

			if (!*p)
				break;
			p++;

			s[n++] = p;
		}

		while (tc > maxlen)
		{
			largest = 0;
			for (lp=0; lp<n; lp++)
				if (c[largest] < c[lp])
					largest = lp;

			c[largest]--;
			tc--;
		}

		p = slarea;
		for (lp=0; lp<n; lp++)
		{
			while (c[lp])
			{
				*p++ = *s[lp]++;
				c[lp]--;
			}
		}
		*p = 0;

		strupr(slarea);
		return;
	}

	char	*t = new char[strlen(tag)+1];

	strcpy(t, tag);

/*	if ( (p = strchr(tag, '_')) != NULL)
	{
		if (p-tag <= 3)
			strcpy(t, p+1);
	}*/

	t[maxlen] = 0;
	strcpy(slarea, t);
	strupr(slarea);

	delete t;
}


int TSearchlight::MakeUniqueSLName(char *tag, char *slarea, char *prefix)
{
	char	s[10];
	int		n, next;

	MakeSLName(tag, s, 8-strlen(prefix));

	strcpy(slarea, prefix);
	strcat(slarea, s);

	n = FindSLArea(slarea);

	if (n == -1)
		return 0;

	if (n == 0)
		return 1;

	if (strlen(slarea) < 8)
		strcat(slarea, "0");

	n = next = strlen(slarea)-1;

	while (FindSLArea(slarea))
	{
		if (slarea[n] >= '0' && slarea[n] <= '8')
			slarea[n]++;
		else
		{
			n = next--;
			slarea[n] = '0';
		}

		if (n == 0)
			return 0;
	}

	return 1;
}
